# ReportDataObject.pm - The Object Class that provides a ReportData Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

#$Id: ReportDataObject.pm,v 1.1.1.1 2001/08/09 19:04:37 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Data::ReportDataObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# global variables
my %regular_cell_types = ( "text" => "Text", "total" => "Total", "compare" => "Compare Rows", "account_type" => "Account Type", "account" => "Account", "sub_account" => "Sub Account", "accounts_of" => "Accounts of Account Type", "sub_accounts_of_type" => "Sub Accounts of Account Type", "sub_accounts_of_account" => "Sub Accounts of Account", "begin_account_type_balance" => "Beginning Account Type Balance", "end_account_type_balance" => "Ending Account Type Balance", "begin_account_balance" => "Beginning Account Balance", "end_account_balance" => "Ending Account Balance", "begin_sub_account_balance" => "Beginning Sub Account Balance", "end_sub_account_balance" => "Ending Sub Account Balance", "line" => "Line" );
my @regular_cell_types = ( "text", "total", "compare", "account_type", "account", "sub_account", "accounts_of", "sub_accounts_of_type", "sub_accounts_of_account", "begin_account_type_balance", "end_account_type_balance", "begin_account_balance", "end_account_balance", "line" ); # "begin_sub_account_balance", "end_sub_account_balance", (to be added later)
my %initial_states = ( "o" => "Open", "c" => "Closed" );
my @initial_states = ( "o", "c" );
my @negative_formats = ( "0", "1", "2" );
my %negative_formats = ( "0" => "<font color=\"red\">(xxx.xx)</font>", "1" => "<font color=\"red\">-</font>xxx.xx", "2" => "xxx.xx<font color=\"red\">-</font>" );
my @fg_colors = ( "black", "blue", "red" );
my %fg_colors = ( "black" => "Black", "blue" => "Blue", "red" => "Red" );
my @bg_colors = ( "white", "yellow", "pink" );
my %bg_colors = ( "white" => "White", "yellow" => "Yellow", "pink" => "Pink" );
my %report_classes = ( "custom" => "Custom", "cash" => "Cash Flow", "income" => "Income Statement", "balance" => "Balance Sheet", "acct_balance" => "Accounts Balance Sheet", "acct_activity" => "Accounts Activity Sheet" );
my @report_classes = ( "custom", "cash", "income", "balance", "acct_balance", "acct_activity" );
my %report_types = ( "r" => "Regular Report", "d" => "Detailed View" );
my @report_types = ( "r", "d" );
my @column_headers = ( "sub_entities", "sub_accounts", "sub_account_types" );
my %column_headers = ( "sub_entities" => "Sub Entities", "sub_accounts" => "Sub Accounts", "sub_account_types" => "Sub Account Types" );
my @font_names = ( "helvetica" );
my %font_names = ( "helvetica" => "Helvetica" );
my @font_sizes = ( "+2", "+1", "+0", "-1", "-2" );
my %font_sizes = ( "+2" => "+2", "+1" => "+1", "+0" => "nothing", "-1" => "-1", "-2" => "-2" );
my %column_names = ( 0 => "Zero", 1 => "One", 2 => "Two", 3 => "Three", 4 => "Four", 5 => "Five", 6 => "Six", 7 => "Seven", 8 => "Eight", 9 => "Nine", 10 => "Ten", 11 => "Eleven", 12 => "Twelve", 13 => "Thirteen", 14 => "Fourteen", 15 => "Fifteen" );
my $max_dates = 4;   # the maximum number of columns that we are currently supporting in the reporting system.
my $min_underline_length =  5;  # the min and max lengths that underlines can range within.
my $max_underline_length = 15;
my %t_or_f = ( 0 => "false", 1 => "true" );  # a mapping that will allow us to map 0 to false and 1 to true for the sql statements.
my %true_false = ( "true" => 1, "false" => 0 );

# new 
# instantiates an instance of a ReportDataObject
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );
  
  $self->{regular_cell_types_hash} = { %regular_cell_types };
  $self->{regular_cell_types_array} = [ @regular_cell_types ];
  $self->{initial_states_hash} = { %initial_states };
  $self->{initial_states_array} = [ @initial_states ];
  $self->{negative_formats_hash} = { %negative_formats };
  $self->{negative_formats_array} = [ @negative_formats ];
  $self->{fg_colors_hash} = { %fg_colors };
  $self->{fg_colors_array} = [ @fg_colors ];
  $self->{bg_colors_hash} = { %bg_colors };
  $self->{bg_colors_array} = [ @bg_colors ];
  $self->{report_classes_hash} = { %report_classes };
  $self->{report_classes_array} = [ @report_classes ];
  $self->{report_types_hash} = { %report_types };
  $self->{report_types_array} = [ @report_types ];
  $self->{column_headers_hash} = { %column_headers };
  $self->{column_headers_array} = [ @column_headers ];
  $self->{font_names_hash} = { %font_names };
  $self->{font_names_array} = [ @font_names ];
  $self->{font_sizes_hash} = { %font_sizes };
  $self->{font_sizes_array} = [ @font_sizes ];
  $self->{column_names_hash} = { %column_names };
  $self->{max_dates} = $max_dates;
  $self->{min_underline_length} = $min_underline_length;
  $self->{max_underline_length} = $max_underline_length;
  $self->{t_or_f} = { %t_or_f };
  $self->{true_false} = { %true_false };

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "ReportDataObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.

  $self->{error} = $error;
  $self->{errorString} = "ReportDataObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

ReportDataObject - Object used to build a ReportData Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Data::ReportDataObject;
  my $obj = Portal::Accounting::Data::ReportDataObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

ReportDataObject is a ReportData class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::Accounting::Data::ReportDataObject
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
