# Variables.pm - The Object Class that provides a Variables Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Variables.pm,v 1.5 2004/02/11 21:06:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Data::Variables;
use Portal::Data::Variables;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

#@ISA = qw(Portal::Data::Variables Exporter AutoLoader);
@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

# global variables
my %investmentActionHash = ('buy' => "Buy",
		    sell => "Sell",
		    spof => "Spin Off",
		    acqs => "Acquisition",
		    sdiv => "Stock Dividend",
		    splt => "Stock Split",
		    sxfr => "Stock Transfer",
		    cxfr => "Cash Transfer",
		    swap => "Stock Swap",
		    memo => "Memo",
		    fee => "Fee",
		    rtcp => "Return of Capital"
		  );
my %investmentNumberHash = ('buy' => "10",
		    sell => "20",
		    spof => "70",
		    acqs => "80",
		    sdiv => "100",
		    splt => "90",
		    sxfr => "30",
		    cxfr => "0",
		    swap => "60",
		    memo => "memo",
		    fee => "40",
		    rtcp => "50"
		  );
my %investmentNumberToCodeHash = ("10" => "buy",
				  "20" => "sell",
				  "70" => "spof",
				  "80" => "acqs",
				  "100" => "sdiv",
				  "90" => "splt",
				  "30" => "sxfr",
				  "0" => "cxfr",
				  "60" => "swap",
				  "memo" => "memo",
				  "40" => "fee",
				  "50" => "rtcp"
				 );
my %journalCodes = (
  "CP" => "Cash Purchases",
  "CSR" => "Cash Reciepts",
  "CDP" => "Credit Purchases",
  "CDR" => "Credit Reciepts",
  "G" => "General",
  "TX" => "Transfers",
  "IS" => "Investments",
  "INV" => "Inventory",
  "S" => "Sales",
  "A" => "Assets",
  );

# new
# instantiates an instance of a Variables
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = (  @_ );

  $self->{investmentActionHash} = \%investmentActionHash;
  $self->{investmentNumberHash} = \%investmentNumberHash;
  $self->{investmentNumberToCodeHash} = \%investmentNumberToCodeHash;
  $self->{journalCodes} = \%journalCodes;
  
  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = "Variables: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.

  $self->{error} = $error;
  $self->{errorString} = "Variables: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

Variables - Object used to build a Variables Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Data::Variables;
  my $obj = Portal::Accounting::Data::Variables->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

VariablesObject is a Variables class.

=head1 Exported FUNCTIONS

  scalar new(void)
    Creates a new instance of the Portal::Accounting::Data::Variables
    object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

=head1 Exported VARIABLES

  hash businessTypes - The number to name of the Business Types.
  hash investmentActionHash - The code to name of Investment Events.
  hash investmentNumberHash - The code to number of Investment Events.
  hash investmentNumberToCodeHash - Number to Code for Investment Events.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
