# Investments.pm - The Object Class that provides a Investments Object
#$Id: Investments.pm,v 1.1 2005/01/26 15:29:38 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::Investments;
use strict;
use Portal::Accounting::Ledger;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Ledger Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.1 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Investments",
  );

=head1 NAME

Investments - Object used to build a Investments for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Investments;
  my $obj = Portal::Accounting::Investments->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This module gives links to the investment subsystems.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Investments module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Investments State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::Investments::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }
  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $l = $self->{langObj};
  my $message = $args{message};
  my $doc = $self->prepDoc;
  my $url = $self->{baseURL};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $j = $l->map("Journal");
  my $p = $l->map("Portfolio");
  my $c = $l->map("Capital Gains/Losses");
  my $w = $l->map("Watchlists");
  my $d = $doc->encodeString(string => "Investments::WatchList");

  $doc->print(<<"ENDOF");
   <a href="$url?app=$self->{app}&state=InvJournal&command=display&ledgerCode=$ledgerCode">$j</a>
   <br> <a href="$url?app=$self->{app}&state=Portfolio&command=display&ledgerCode=$ledgerCode">$p</a>
   <br> <a href="$url?app=$self->{app}&state=CGL&command=display&ledgerCode=$ledgerCode">$c</a>
   <br> <a href="$url?app=$self->{app}&state=$d&command=display&ledgerCode=$ledgerCode">$w</a>
ENDOF

  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut