# JournalEntry.pm - The Object Class that provides a JournalEntry Object
#$Id: JournalEntry.pm,v 1.6 2004/11/02 20:34:57 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::JournalEntry;
use strict;
use HTMLObject::Base;
use HTMLObject::Normal;
use Portal::Accounting::Objects::HTMLJournalEntry;
use Portal::Accounting::DB::JournalEntry;
use Portal::Accounting::Ledger;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

use Date::Calc qw(Today);

require Exporter;

@ISA = qw(Portal::Accounting::Ledger Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "General Ledger",
  demo => "Demo Data",
  delete => "Delete",
  check => "Check",
  saveCheck => "Save Check",
  deposit => "Deposit",
  saveDeposit => "Save Deposit",
  doubleEntry => "Double Entry",
  saveDoubleEntry => "Save Double Entry",
  );

=head1 NAME

JournalEntry

=head1 SYNOPSIS

  use Portal::Accounting::JournalEntry;
  my $obj = Portal::Accounting::JournalEntry->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This set of screens will hold the forms for data Entry in the system.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion, #!!!portal args??)

 Creates a new instance of the Portal::Accounting::JournalEntry module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the JournalEntry State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( release => "", DBVersion => "", @_ );

  $self->{commands} = \%commands;

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::JournalEntry::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $message = $args{message};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};

  my $doc = $self->prepDoc;
  my $url = $self->{baseURL};

  my $demo = $self->langObj->map("Demo Data");
  my $Check = $self->langObj->map("Check");
  my $Check = $self->langObj->map("Check");
  my $Deposit = $self->langObj->map("Deposit");
  my $Double = $self->langObj->map("Double Entry");
  my $Credit = $self->langObj->map("Credit Purchase");
  my $CreditR = $self->langObj->map("Credit Receipt");
  my $Search = $self->langObj->map("Search");

  my $s = $self->{input}->{state};
  my $a = $self->{input}->{app};
  my $l = $self->{input}->{ledgerCode};
  $doc->print(<<"EOFO");
<a href="?app=$a&state=$s&command=display&ledgerCode=$l">$Search</a><br>
<a href="?app=$a&state=$s&command=check&ledgerCode=$l">$Check</a><br>
<a href="?app=$a&state=$s&command=deposit&ledgerCode=$l">$Deposit</a><br>
<a href="?app=$a&state=$s&command=doubleEntry&ledgerCode=$l">$Double</a><br>
<a href="?app=$a&state=$s&command=creditPurchase&ledgerCode=$l">$Credit</a><br>
<a href="?app=$a&state=$s&command=creditReciept&ledgerCode=$l">$CreditR</a><br>
<a href="?app=$a&state=$s&command=demo&ledgerCode=$l">$demo</a><br>
EOFO
  return $doc;
}

sub c_saveCheck
{
  my $self = shift;
  return $self->c_check;
}

sub c_check
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->error)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }
  $doc->setTitle($phrases->map("Edit Check").":$self->{input}->{ledger}");
  $doc->setFocus("body");
  my $je = Portal::Accounting::Objects::HTMLJournalEntry->new(langObj => $self->{langObj});
  if ($je->error)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
  }
  else
  {
    my $empty = "n";
    if ($self->{input}->{command} eq "check")
    {  $empty = "y"; }
    my $form = $je->printForm(empty => $empty, input => $self->{input}, command => "saveCheck");
    if ($je->error)
    {
      $form = $je->errorMessage;
    }
    $doc->print($form);
  }

  return $doc;
}

sub c_demo
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $lc = $self->{input}->{ledgerCode};
  my $year = 2004;
  my $db  = Portal::Accounting::DB::JournalEntry->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->error)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }
  $doc->setTitle($phrases->map("Demo Data").":$self->{input}->{ledger}");
  $doc->setFocus("body");
  $doc->print("<h4>".$phrases->map("Demo Data")." : $self->{input}->{ledgerCode}</h4>");
  if (!$self->{userObj}->{admin})
  {
    $doc->print("Only an administrator can enter demo data!<br>\n");
    return $doc;
  }
  if ($self->{input}->{confirmDemo} ne "1")
  {
    $doc->print(<<"ENDOF");
 Click this button if you are sure that you want to enter demo data into these books.<br>
  <form>
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="demo">
    <input type="hidden" name="ledgerCode" value="$lc">
    <input type="hidden" name="confirmDemo" value="1">
    <input type="submit" name="demo" value="X">
  </form>
ENDOF
    return $doc;
  }
  my $bankSql = "SELECT MIN(number) AS bank";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 5) AS expense";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 4) AS revenue";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 2) AS liability";
  $bankSql .= ", (SELECT MIN(account_tb.number) FROM account_tb,acct_type_tb WHERE acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = 3) AS equity";
  $bankSql .= ", (SELECT working_year('$lc')) AS year";
  $bankSql .= "  FROM account_tb WHERE type = 0";
  my $result = $self->{dbHandle}->read($bankSql);
  if ($self->{dbHandle}->error)
  {
    $doc->print($self->{dbHandle}->errorMessage);
    return $doc;
  }
  my $info;
  if (not $info = $result->fetchrow_hashref)
  {
    $doc->print($self->{dbHandle}->errorMessage);
    return $doc;
  }
  $year = $info->{year};
  if ($year !~ /^\d{4}$/) { my @year = Today; $year = $year[0];}
  my $expense = $info->{expense};
  my $revenue = $info->{revenue};
  my $bank = $info->{bank};
  my $equity = $info->{equity};
  my $liability = $info->{liability};
#setup objects
  #loop 12 times for inserting 12 months.  start with 2 bc 1 is already set
  for (my $month = 1; $month < 13; $month++)
  {
    my $elec = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj});
    my $water = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj});
    my $pay = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj});
  #populate objects
    $elec->populate(
        ledgerCode => $self->{input}->{ledgerCode},
        payeePayer => 'Some Electric Co',
        remember => 'false',
        date => "$year-$month-10",
        memo => "month $month",
    );
    $elec->addRow(
      comment => "month $month",
      accountNum => $expense,
      amount => "50.00",
    );
    $elec->addRow(
      comment => "month $month",
      accountNum => $bank,
      amount => "-50.00",
    );
    if (! $elec->isValid)
    {
      $doc->print($elec->errorMessage);
      $doc->print("<br>0: ".$elec->{rowObjects}[0]->dorc."<br>1: ".$elec->{rowObjects}[1]->{dorc}."<br>");
      $doc->print("0: ".$elec->{rowObjects}[0]->{amount}."<br>1: ".$elec->{rowObjects}[1]->{amount}."<br>");
      $doc->print("0: ".$elec->{rowObjects}[0]->{d_amount}."<br>1: ".$elec->{rowObjects}[1]->{d_amount}."<br>");
      $doc->print("0: ".$elec->{rowObjects}[0]->{c_amount}."<br>1: ".$elec->{rowObjects}[1]->{c_amount}."<br>");
      $doc->print("0: ".$elec->{rowObjects}[0]->accountNum."<br>1: ".$elec->{rowObjects}[1]->accountNum."<br>");
      return $doc;
    }

    #water object
    $water->populate(
        ledgerCode => $self->{input}->{ledgerCode},
        payeePayer => 'My Water Co',
        remember => 'false',
        date => "$year-$month-15",
        memo => "month $month",
    );
    $water->addRow(
      comment => "month $month",
      accountNum => $expense,
      amount => "10.00",
    );
    $water->addRow(
      comment => "month $month",
      accountNum => $bank,
      amount => "-10.00",
    );
    if (! $water->isValid)
    {
      $doc->print($water->errorMessage);
      $doc->print("<br>0: ".$water->{rowObjects}[0]->dorc."<br>1: ".$water->{rowObjects}[1]->{dorc}."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->{amount}."<br>1: ".$water->{rowObjects}[1]->{amount}."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->{d_amount}."<br>1: ".$water->{rowObjects}[1]->{d_amount}."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->{c_amount}."<br>1: ".$water->{rowObjects}[1]->{c_amount}."<br>");
      $doc->print("0: ".$water->{rowObjects}[0]->accountNum."<br>1: ".$water->{rowObjects}[1]->accountNum."<br>");
      return $doc;
    }

    #pay object
    $pay->populate(
        ledgerCode => $self->{input}->{ledgerCode},
        payeePayer => 'Revenue',
        remember => 'false',
        date => "$year-$month-1",
        memo => "month $month",
    );
    $pay->addRow(
      comment => "month $month",
      accountNum => $revenue,
      amount => "1000.00",
    );
    $pay->addRow(
      comment => "month $month",
      accountNum => $bank,
      amount => "1000.00",
    );
    if (! $pay->isValid)
    {
      $doc->print($pay->errorMessage);
      $doc->print("<br>0: ".$pay->{rowObjects}[0]->dorc."<br>1: ".$pay->{rowObjects}[1]->{dorc}."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->{amount}."<br>1: ".$pay->{rowObjects}[1]->{amount}."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->{d_amount}."<br>1: ".$pay->{rowObjects}[1]->{d_amount}."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->{c_amount}."<br>1: ".$pay->{rowObjects}[1]->{c_amount}."<br>");
      $doc->print("0: ".$pay->{rowObjects}[0]->accountNum."<br>1: ".$pay->{rowObjects}[1]->accountNum."<br>");
      return $doc;
    }

    my $ref;
    my $r;

    #insert current data into DB first
    $ref = $db->insert(obj => $elec, commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>".$db->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$elec->date."<br>");
    $ref = $db->insert(obj => $water, commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>".$db->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$water->date."<br>");
    $ref = $db->insert(obj => $pay, commit => 1);
    if (not defined $ref)
    {
      $doc->print("<h4>Error Occurred while inserting Journal Entry Object</h4>".$db->errorMessage."<br>\n");
      return $doc;
    }
    $doc->print($ref.":".$pay->date."<br>");
  }
  $doc->print("Data Entered successfully for $year<br>\n");
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
