# CloseOut.pm - The Object Class that provides a CloseOut Object
#$Id: CloseOut.pm,v 1.9 2004/09/02 21:43:41 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::CloseOut;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

=head1 NAME

CloseOut - Object used to build a CloseOut Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CloseOut;
  my $CloseOutObj = Portal::Accounting::Objects::CloseOut->new(   );
  if ($CloseOutObj->didErrorOccur())
  {
    die $CloseOutObj->errorMessage();
  }

=head1 DESCRIPTION

CloseOut is a CloseOut class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::CloseOut.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do anything else you might need to do.
  my $phrases = $self->{langObj};
  $self->{data} = {
    "account" => { -Label => "", -Value => "",
      -Type => "select",
      },
    "date" => { -Label => $phrases->map("close_lock"),
      -Type => "date-picker"
      },
    "businessType" => { -Label => $phrases->map("Business Type"), -Type => "text", -ReadOnly => 1 },      
    "command" => { -Type => "hidden", -Value => "closeOut" },      
    "ledgerCode" => { -Type => "hidden", -Value => "" },      
    "app" => { -Type => "hidden", -Value => "Accounting" },      
    "state" => { -Type => "hidden", -Value => "Ledger" },      
    "lastClose" => { -Label => $phrases->map("Last Close Out"), -Type => "text", -ReadOnly => 1 },      
    "dates" => { -Label => "",
      -Type => "select", size => 4, onchange => "enterDate(this);"      
      },
  };
  
  my $message = $phrases->map("closeoutExplain")."<br><small>".$phrases->map("closeNote")."</small>";
  $self->{javascript} = <<"ENDOF";
  function enterDate(me)
  {
    if (me.selectedIndex == -1)
    {
      return;
    }
    me.form.date.value = me.options[me.selectedIndex].value;
  }
ENDOF
  my $submit = $phrases->map("Submit");
  $self->{template} = <<"ENDOF";
  <p><form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #LABEL=businessType#: #FIELD=businessType#<br>
  #LABEL=lastClose#: #FIELD=lastClose#</p>
  <p>$message<br>
  <table cellpadding="15">
    <tr valign="top">
      <td>
        #FIELD=account#<br>
        #LABEL=date#<br>
        #FIELD=date#<br>
      </td>
      <td>
        #FIELD=dates#<br>
      </td>
    </tr>
  </table>
  <p><input type="submit" value="$submit"><input type="reset"></p>
  </p>
  </form>
ENDOF

  $self->{profile} = {
    required => [ qw( account date ) ],
    constraints  =>
        {
            date   => qr/^\d{4}-\d{2}-\d{2}$/,
            account  => qr/^3\d+$/,
        },
  };
  
  return $self;
}

=item bool populate(date => , dates =>, year => , lastClose => , account => , businessType => );

  this method inserts the data given into the object.
  account is null separated list of account numbers to use for equity account
  dates is an array ref to array of dates
  
=cut
sub populate
{
  my $self = shift;
  my %args = (businessType => "", year => "", date => "", lastClose => "", accounts => undef, dates => undef, 
    ledgerCode => $self->{data}->{ledgerCode}->{-Value},
    app => $self->{data}->{app}->{-Value},
    state => $self->{data}->{state}->{-Value},
    command => $self->{data}->{command}->{-Value},
    account => $self->{data}->{account}->{-Value},
    @_);
  $self->{data}->{ledgerCode}->{-Value} = $args{ledgerCode};
  $self->{data}->{app}->{-Value} = $args{app};
  $self->{data}->{state}->{-Value} = $args{state};
  $self->{data}->{command}->{-Value} = $args{command};
  $self->{data}->{date}->{year} = $args{year};
  $self->{data}->{account}->{-Options} = $args{accounts};
  $self->{data}->{account}->{-Value} = $args{account};
  $self->{data}->{date}->{-Value} = $args{date};
  $self->{data}->{lastClose}->{-Value} = $args{lastClose};
  $self->{data}->{businessType}->{-Value} = $args{businessType};
  
  $self->{data}->{dates}->{-Options}->{names} = [ ];
  $self->{data}->{dates}->{-Options}->{values} = [ ];
  foreach my $date (@{$args{dates}}) 
  {
    push @{$self->{data}->{dates}->{-Options}->{names}}, $date;
    push @{$self->{data}->{dates}->{-Options}->{values}}, $date;
  }
  
  return 1;
  #return $self->Portal::Accounting::Objects::CloseOut::isValid;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ( $self->{data}->{date}->{-Value} ne "" && $self->{data}->{date}->{-Value} !~ /^\d{4}-\d{2}-\d{2}$/)
  {
    $self->invalid("date");
  }
  if ( $self->{data}->{account}->{-Value} !~ /^3\d+(\03\d+)*$/)
  {
    $self->invalid("account");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{data}->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name}->{-Value} = shift;
  }
  else
  {
    return $self->{$name}->{-Value};
  }
}

=item string print()

  This routine returns a text string that represents the data in this object.

=cut

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
}

=item string printHTML()

  This routine returns a string of html that represents this object.

=cut

=item string printForm( prefix => )

  This routine returns xhtml form elements that can be used to edit this object.
  A prefix can be used to uniquely identify this form from others of the same 
  type.  The names of form elements can be obtained in a hash:

  use Portal::Accounting::Objects::Base;
  my %elements = %Portal::Accounting::Objects::CloseOut::Elements;

  !!Make a copy of the hash as shown above--NOT a refence.
  You would generally place your input from the user within the Elements hash.
  Then you can pass the Elements hash to the new method
  of a CloseOut object instead of individual arguments.

  foreach my $prefix (%prefixes)
    foreach $index (key %Elements)
      $tempHash{$index} = $input->{$prefix . $index}
    my object = Portal::Accounting::Objects::CloseOut->new(%tempHash);

=cut

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
