# JournalEntryRow.pm - Contains the data for a debitcredit_tb table row.
# Created by Jason A. Pattie, 11/07/2000.
#$Id: JournalEntryRow.pm,v 1.4 2004/10/29 21:45:52 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::JournalEntryRow;
use strict;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;
@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.4 $';

# new
# instantiates an instance of the JournalEntryRow for the Accounting Package
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  return $self;
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  if (!$self->SUPER::isValid())
  {
    $self->prefixError("");
    return 0;
  }

  if ($self->{date} eq "")
  {
    $self->invalid("row date", $self->{date});
  }
  if (not defined $self->{comment})
  {
    $self->{comment} = "";
  }
  if (not defined $self->{subEntity})
  {
    $self->{subEntity} = "";
  }
  if ($self->{accountNum} !~ /^[1-5]\d+$/)
  {
    $self->invalid("accountNum", $self->{accountNum});
  }
  if (not defined $self->{subAccountNum})
  {
    $self->{subAccountNum} = -1;
  }
  if ($self->{'subAccountNum'} !~ /^\d+$/ && $self->{'subAccountNum'} != -1)
  {
    $self->invalid("subAccountNum", $self->{subAccountNum});
  }
  if ($self->{dorc} ne "true" and $self->{dorc} ne "false")
  {
    $self->invalid("dorc", $self->{dorc});
  }
  if ($self->{amount} !~ /^-?([0-9]*).([0-9]{2})$/)
  {
    $self->invalid("amount", $self->{amount});
  }
  if ($self->{d_amount} !~ /^-?([0-9]*).([0-9]{2})$/)
  {
    $self->invalid("d_amount", $self->{amount});
  }
  if ($self->{cleared} !~ /[_xr]/)
  {
    $self->{cleared} = '_';
  }

  if ($self->{error})
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

sub populate
{
  my $self = shift;
  my %args = ( date => "", comment => '', subEntity => '', accountNum => "", subAccountNum => -1,
               amount => "", cleared => '_', @_ );

  $self->{date}          = $args{'date'};
  $self->{comment}       = $args{'comment'};
  $self->{subEntity}     = $args{'subEntity'};
  $self->{accountNum}    = $args{'accountNum'};
  $self->{subAccountNum} = $args{'subAccountNum'};
  $self->{cleared}       = $args{'cleared'};
  $self->{d_amount}      = $args{'amount'};
  $self->{amount}        = $args{'amount'};
  $self->{c_amount}      = 0.0;
  if ($self->{amount} < 0) {  $self->{amount} = $self->{amount} - $self->{amount} - $self->{amount};  }  #faster than *= -1
  if ($self->{accountNum} =~ /^1/)
  {
    if ($args{amount} < 0)  #decreasing asset
    {
      $self->{dorc} = 'false';
      $self->{c_amount}  = $self->{'amount'};
    }
    else  #increasing asset decreases cash
    {
      $self->{dorc} = 'true';
      $self->{c_amount}  -= $self->{'amount'};
    }
  }
  elsif ($self->{accountNum} =~ /^2/)
  {
    if ($args{amount} < 0) #decreasing liability uses cash
    {
      $self->{dorc} = 'true';
      $self->{c_amount}  -= $self->{'amount'};
    }
    else  #increasing liability gets cash
    {
      $self->{dorc} = 'false';
      $self->{c_amount}  = $self->{'amount'};
    }
  }
  elsif ($self->{accountNum} =~ /^3/)
  {
    if ($args{amount} < 0) #decreasing equity uses cash
    {
      $self->{dorc} = 'true';
      $self->{c_amount}  -= $self->{'amount'};
    }
    else  #increasing equity gets cash
    {
      $self->{dorc} = 'false';
      $self->{c_amount}  = $self->{'amount'};
    }
  }
  elsif ($self->{accountNum} =~ /^4/)  #same as 3
  {
    if ($args{amount} < 0)  #decreasing revenues uses cash
    {
      $self->{dorc} = 'true';
      $self->{c_amount}  -= $self->{'amount'};
    }
    else  #increasing revenues increases cash
    {
      $self->{dorc} = 'false';
      $self->{c_amount}  = $self->{'amount'};
    }
  }
  elsif ($self->{accountNum} =~ /^5/)  #same as 1
  {
    if ($args{amount} < 0) #negative expenses refund cash
    {
      $self->{dorc} = 'false';
      $self->{c_amount}  = $self->{'amount'};
    }
    else  #expenses use cash
    {
      $self->{dorc} = 'true';
      $self->{c_amount}  -= $self->{'amount'};
    }
  }
  else { $self->prefixError("invalid account") ; return undef; }

  return $self->isValid;
}

sub setDate
{
  my $self = shift;
  my $date = shift;
  if ($date =~ /^(\d{4})[\-\\\/](\d{1,2})[\-\\\/](\d{1,2})$/)
  {
    $date = $1 ."-". sprintf("%02d", $2) ."-". sprintf("%02d", $3);
  }
  else
  {
    $self->error("Invalid date: $date<br>\n");
    return 0;
  }

  $self->{date} = $date;
  foreach my $row (@{$self->{rowObjects}})
  {
    $row->{date} = $date;
  }
  return 1;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  my %mapping = ("comment" => "comment", "accountCode" => "accountNum", "date" => "rowDate",
                 "subAccountCode" => "subAccountNum", "dorc" => "dorc", "amount" => "amount",
                 "subEntityCode" => "subEntity", "cleared" => "cleared");

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
    $result .= $spaceStr . "<line>\n";
    my @tags = qw{dorc accountCode amount subAccountCode comment subEntityCode cleared};
    push @tags, "date" if ($fullXML);
    foreach my $name (@tags)
    {
      $result .= $spaceStr . "  <$name>" . $self->{$mapping{$name}} . "</$name>\n";
    }
    $result .= $spaceStr . "</line>\n";
  }
  else
  {
    $result .= "Data not valid!\n\n";
    $result .= $self->errorMessage();
    die $result;
  }

  return $result;
}

1;
__END__

=head1 NAME

JournalEntryRow - Object that contains a row of data used to build a JournalEntry in the accounting database.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::JournalEntryRow;
  my $transrowobj = JournalEntryRow->new(rowDate => '12-14-1999',
                             comment => '',
                             subEntity => '',
                             accountNum => 1001,
                             subAccountNum => -1,
                             dorc => 'true',
                             amount => '2579.63',
                             cleared => 'r');
  if ($transrowobj->error())
  {
    die $transrowobj->errorMessage();
  }

  # this object is generally used to build an array passed to
  # the constructor of a JournalEntryObject
  my @rowObjectArray;
  $rowObjectArray[0] = $transrowobj;
  $rowObjectArray[1] = $transrowobj;
  $rowObjectArray[1]->dorc('false');

  # check for validness of row objects values
  # before adding to JournalEntryObject
  if ($rowObjectArray[1]->isValid())
  {
    ...
  }
  else
  {
    die $rowObjectArray[1]->errorMessage();
  }

=head1 DESCRIPTION

JournalEntryRow is used as a container class that is incorporated
into the JournalEntryObject class.  This class is used to build an array
that is given to the JournalEntryObject class.

=head1 Exported FUNCTIONS

  scalar new(rowDate, comment, subEntity, accountNum, subAccountNum, dorc,
             amount, cleared)
    Creates a new instance of the Portal::Accounting::Objects::JournalEntryRow
    and validates the values contained in it.

  scalar error(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar error()
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function will be called
    automatically when embedded in a JournalEntryObject.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1), JournalEntryObject(3)

=cut
