# RegularReportDataObject.pm - The Object Class that provides a RegularReportData Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: RegularReportDataObject.pm,v 1.1.1.1 2001/08/09 19:04:51 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::RegularReportDataObject;
use strict;
use Portal::Accounting::Data::ReportDataObject;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a RegularReportDataObject
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( id => 0, index => 0, cell_text => "", cell_type => "text", item_name => "", fg_color => "black", bg_color => "white", display_level => 0, initial_state => "o", negative_format => 0, other_attributes => "", @_ );
  
  $self->{index} = $args{index};
  $self->{cell_text} = $args{cell_text};
  $self->{cell_type} = $args{cell_type};
  $self->{item_name} = $args{item_name};
  $self->{fg_color} = $args{fg_color};
  $self->{bg_color} = $args{bg_color};
  $self->{display_level} = $args{display_level};
  $self->{initial_state} = $args{initial_state};
  $self->{negative_format} = $args{negative_format};
  $self->{other_attributes} = $args{other_attributes};
  
  $self->{data_obj} = Portal::Accounting::Data::ReportDataObject->new;
  
  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "RegularReportDataObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.
  if ($self->{id} < 0)
  {
    $error = 1;
    $errorString .= "Id = '$self->{id}' is invalid!<br>\n";
  }
  if ($self->{index} < 0)
  {
    $error = 1;
    $errorString .= "Index = '$self->{index}' is invalid!<br>\n";
  }
  if (not exists $self->{data_obj}->regular_cell_types_hash->{$self->{cell_type}})
  {
    $error = 1;
    $errorString .= "Cell Type = '$self->{cell_type}' is invalid!<br>\n";
    goto QuickExit; # leave early as following checks need to rely on this value being valid.
  }
  if (not exists $self->{data_obj}->fg_colors_hash->{$self->{fg_color}})
  {
    $error = 1;
    $errorString .= "ForeGround Color = '$self->{fg_color}' is invalid!<br>\n";
  }
  if (not exists $self->{data_obj}->bg_colors_hash->{$self->{bg_color}})
  {
    $error = 1;
    $errorString .= "BackGround Color = '$self->{bg_color}' is invalid!<br>\n";
  }
  if ($self->{cell_type} eq "line")
  {
    if (length $self->{cell_text} > 0)
    {
      $error = 1;
      $errorString .= "Cell Text = '$self->{cell_text}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (length $self->{item_name} > 0)
    {
      $error = 1;
      $errorString .= "Item Name can not be specified.  '$self->{item_name}' is invalid!<br>\n";
    }
    if ($self->{display_level} != 0)
    {
      $error = 1;
      $errorString .= "Display Level = '$self->{display_level}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if ($self->{initial_state} ne "o" || (not exists $self->{data_obj}->initial_states_hash->{$self->{initial_state}}))
    {
      $error = 1;
      $errorString .= "Initial State = '$self->{initial_state}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (length $self->{other_attributes} > 0)
    {
      $error = 1;
      $errorString .= "Other Attributes = '$self->{other_attributes}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if ($self->{negative_format} != 0)
    {
      $error = 1;
      $errorString .= "Negative Format = '$self->{negative_format}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
  }
  elsif ($self->{cell_type} eq "text" or $self->{cell_type} eq "total" or $self->{cell_type} eq "compare")
  {
    if ($self->{cell_type} ne "compare")
    {
      if (length $self->{cell_text} == 0)
      {
        $error = 1;
        $errorString .= "Cell Text must be specified for Cell Type = '$self->{cell_type}'!<br>\n";
      }
    }
    if (length $self->{item_name} > 0)
    {
      $error = 1;
      $errorString .= "Item Name can not be specified.  '$self->{item_name}' is invalid!<br>\n";
    }
    if ($self->{cell_type} eq "text")
    {
      if (length $self->{other_attributes} > 0)
      {
        $error = 1;
        $errorString .= "Other Attributes = '$self->{other_attributes}' can not be specified for Cell Type = '$self->{cell_type}'!<br>\n";
      }
      if ($self->{negative_format} != 0)
      {
        $error = 1;
        $errorString .= "Negative Format = '$self->{negative_format}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
      }
    }
    else
    {
      if (length $self->{other_attributes} == 0)
      {
        $error = 1;
        $errorString .= "Other Attributes must be specified for Cell Type = '$self->{cell_type}'!<br>\n";
      }
      if (not exists $self->{data_obj}->negative_formats_hash->{$self->{negative_format}})
      {
        $error = 1;
        $errorString .= "Negative Format = '$self->{negative_format}' is invalid!  Cell Type = '$self->{cell_type}'.<br>\n";
      }
    }
    if (length $self->{item_name} > 0)
    {
      $error = 1;
      $errorString .= "Item Name can not be specified.  '$self->{item_name}' is invalid!<br>\n";
    }
    if ($self->{display_level} < 0 || $self->{display_level} > 2)
    {
      $error = 1;
      $errorString .= "Display Level = '$self->{display_level}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (not exists $self->{data_obj}->initial_states_hash->{$self->{initial_state}})
    {
      $error = 1;
      $errorString .= "Initial State = '$self->{initial_state}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
  }
  else
  {
    if (length $self->{cell_text} > 0)
    {
      $error = 1;
      $errorString .= "Cell Text = '$self->{cell_text}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (length $self->{item_name} == 0)
    {
      $error = 1;
      $errorString .= "Item Name must be specified for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if ($self->{display_level} < 0 || $self->{display_level} > 2)
    {
      $error = 1;
      $errorString .= "Display Level = '$self->{display_level}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (not exists $self->{data_obj}->initial_states_hash->{$self->{initial_state}})
    {
      $error = 1;
      $errorString .= "Initial State = '$self->{initial_state}' is invalid to be set for Cell Type = '$self->{cell_type}'!<br>\n";
    }
    if (not exists $self->{data_obj}->negative_formats_hash->{$self->{negative_format}})
    {
      $error = 1;
      $errorString .= "Negative Format = '$self->{negative_format}' is invalid!  Cell Type = '$self->{cell_type}'.<br>\n";
    }
  }

QuickExit:
  $self->{error} = $error;
  $self->{errorString} = "RegularReportDataObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

RegularReportDataObject - Object used to build a RegularReportData Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::RegularReportDataObject;
  my $obj = Portal::Accounting::Objects::RegularReportDataObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

RegularReportDataObject is a RegularReportData class.

=head1 Exported FUNCTIONS

  scalar new(id, index, cell_type, cell_text, fg_color, bg_color,
             display_level, initial_state, negative_format,
             other_attributes)
    Creates a new instance of the
    Portal::Accounting::Objects::RegularReportDataObject object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
