# SecurityEventObject.pm - The Object Class that provides a Security Event Object
# that contains the necessary data values based on the event code
# Created by Jason A. Pattie, 11/10/2000.
#$Id: SecurityEventObject.pm,v 1.2 2003/10/01 20:09:32 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SecurityEventObject;
use strict;
use Portal::Accounting::Data::Variables;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a SecurityEventObject
# optional: stocks, accounts (hash refs that lookup number to name)
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my $errorHeader = "SecurityEventObject->new(): ";
  my %args = (eventCode => undef, masterEventRef => undef, calcDate => undef, transDate => undef,
              counter => 0, counter2 => 0,
              subEntity => undef, accountNum => undef, newAccountNum => undef, oldAccountNum => undef, memoText => undef,
              subAccountNum => undef, subAccountNum2 => undef, lot => undef, newLot => undef, oldLot => undef,
              amount => undef, newShares => undef, oldShares => undef, shares => undef,
              commission => undef, costPercentage => undef, price => undef,
              ledgerCode => undef, eventRefNum => 0, @_ );

  $self->{error} = 0;  # nothing wrong yet.
  $self->{errorString} = "";

  $self->{eventCode}      = $args{'eventCode'};
  $self->{masterEventRef} = $args{'masterEventRef'};
  $self->{calcDate}       = $args{'calcDate'};
  $self->{transDate}      = $args{'transDate'};
  $self->{counter}        = $args{'counter'};
  $self->{counter2}       = $args{'counter2'};
  $self->{subEntity}      = $args{'subEntity'};
  $self->{accountNum}     = $args{'accountNum'};
  $self->{newAccountNum}  = $args{'newAccountNum'};
  $self->{oldAccountNum}  = $args{'oldAccountNum'};
  $self->{memoText}       = $args{'memoText'};
  $self->{subAccountNum}  = $args{'subAccountNum'};
  $self->{subAccountNum2} = $args{'subAccountNum2'};
  $self->{lot}            = $args{'lot'};
  $self->{newLot}         = $args{'newLot'};
  $self->{oldLot}         = $args{'oldLot'};
  $self->{amount}         = $args{'amount'};
  $self->{newShares}      = $args{'newShares'};
  $self->{oldShares}      = $args{'oldShares'};
  $self->{shares}         = $args{'shares'};
  $self->{commission}     = $args{'commission'};
  $self->{costPercentage} = $args{'costPercentage'};
  $self->{price}          = $args{'price'};
  $self->{ledgerCode}     = $args{'ledgerCode'};
  $self->{eventRefNum}    = $args{'eventRefNum'};
  
  return $self;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  my $errorHeader = "SecurityEventObject->isValid(): ";
  my @validate_list = ();

  if ($self->{eventCode} eq "10") # buy
  {
    @validate_list = ("calcDate", "transDate", "counter", "subEntity", "accountNum", "subAccountNum", "lot", "price", "shares", "commission", "ledgerCode", "eventRefNum");
    # local event only
    if ($self->{ledgerCode} eq "*")
    {
      # clear ledgerCode, invalidating the object
      $self->{ledgerCode} = "";
    }
  }
  elsif ($self->{eventCode} eq "20") # sell
  {
    @validate_list = ("masterEventRef", "calcDate", "transDate", "counter", "accountNum", "subAccountNum", "lot", "price", "shares", "commission", "ledgerCode", "eventRefNum");
    # local event only
    if ($self->{ledgerCode} eq "*")
    {
      # clear ledgerCode, invalidating the object
      $self->{ledgerCode} = "";
    }
  }
  elsif ($self->{eventCode} eq "30") # stock transfer
  {
    @validate_list = ("calcDate", "transDate", "counter", "subEntity", "oldAccountNum", "newAccountNum", "subAccountNum", "newLot", "oldLot", "shares", "ledgerCode", "eventRefNum");
    # local event only
    if ($self->{ledgerCode} eq "*")
    {
      # clear ledgerCode, invalidating the object
      $self->{ledgerCode} = "";
    }
  }
  elsif ($self->{eventCode} eq "40") # fee
  {
    @validate_list = ("calcDate", "transDate", "counter", "accountNum", "subAccountNum", "lot", "amount", "ledgerCode", "eventRefNum");
    # local event only
    if ($self->{ledgerCode} eq "*")
    {
      # clear ledgerCode, invalidating the object
      $self->{ledgerCode} = "";
    }
  }
  elsif ($self->{eventCode} eq "50") # return of capital
  {
    @validate_list = ("calcDate", "transDate", "counter", "subAccountNum", "amount", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "60") # stock swap
  {
    @validate_list = ("calcDate", "transDate", "counter", "counter2", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "costPercentage", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "70") # spin off
  {
    @validate_list = ("calcDate", "transDate", "counter", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "costPercentage", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "80") # acquisition
  {
    @validate_list = ("calcDate", "transDate", "counter", "subAccountNum", "subAccountNum2", "oldShares", "newShares", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "90") # stock split
  {
    @validate_list = ("calcDate", "transDate", "counter", "subAccountNum", "oldShares", "newShares", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "100") # stock dividend
  {
    @validate_list = ("calcDate", "transDate", "counter", "subAccountNum", "oldShares", "newShares", "ledgerCode", "eventRefNum");
    # global event; no matter what the ledgerCode was before, modify it to be an asterisk (*)
    $self->{ledgerCode} = "*";
  }
  elsif ($self->{eventCode} eq "memo")
  {
    @validate_list = ("calcDate", "transDate", "counter", "memoText", "subAccountNum", "ledgerCode", "eventRefNum");
    # local event only
    if ($self->{ledgerCode} eq "*")
    {
      # clear ledgerCode, invalidating the object
      $self->{ledgerCode} = "";
    }
  }
  else
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Invalid eventCode assigned to this object (" . $self->{eventCode} . ")!";
    return 0;
  }
  # run through the validate list and validate each necessary field value
  for (my $validatecount=0; $validatecount < scalar @validate_list; $validatecount++)
  {
    if (not defined $self->{$validate_list[$validatecount]})
    {
      $self->{error} = 1;
      $self->{errorString} = $errorHeader . "The $validate_list[$validatecount] must be specified!";
      return 0;
    }
    if ($validate_list[$validatecount] eq "calcDate")
    {
      if ($self->{calcDate} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty calc date!  The calc date must be specified.";
        return 0;
      }
      if ($self->{calcDate} !~ /^([0]?[1-9]|[1][0-2])[-\/]([0]?[1-9]|[1-2][0-9]|[3][0-1])[-\/]([0-9]{4})$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid date format or value (" . $self->{calcDate} . ")!  The calc date must be of the form MM/DD/YYYY or MM-DD-YYYY.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "transDate")
    {
      if ($self->{transDate} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty trans date!  The trans date must be specified.";
        return 0;
      }
      if ($self->{transDate} !~ /^([0]?[1-9]|[1][0-2])[-\/]([0]?[1-9]|[1-2][0-9]|[3][0-1])[-\/]([0-9]{4})$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid date format or value (" . $self->{transDate} . ")!  The trans date must be of the form MM/DD/YYYY or MM-DD-YYYY.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "counter")
    {
      if ($self->{counter} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid counter number (" . $self->{counter} . ")!  The value does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "counter2")
    {
      if ($self->{counter2} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid counter2 number (" . $self->{counter2} . ")!  The value does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "accountNum")
    {
      if ($self->{accountNum} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid account number (" . $self->{accountNum} . ")!  The account number does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "newAccountNum")
    {
      if ($self->{newAccountNum} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid new account number (" . $self->{newAccountNum} . ")!  The new account number does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "oldAccountNum")
    {
      if ($self->{oldAccountNum} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid old account number (" . $self->{oldAccountNum} . ")!  The old account number does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "memoText")
    {
      if ($self->{memoText} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty memo text!  The memo text must be specified.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "subAccountNum")
    {
      if ($self->{subAccountNum} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty sub-account number!  The sub-account number must be specified.";
        return 0;
      }
      if ($self->{subAccountNum} !~ /^-?[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "The sub-account number does not appear to be numeric (" . $self->{subAccountNum} . ")!";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "subAccountNum2")
    {
      if ($self->{subAccountNum2} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty sub-account number 2!  The sub-account number 2 must be specified.";
        return 0;
      }
      if ($self->{subAccountNum2} !~ /^-?[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "The sub-account number 2 does not appear to be numeric (" . $self->{subAccountNum2} . ")!";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "lot")
    {
      if ($self->{lot} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid lot (" . $self->{lot} . ")!  Lot does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "newLot")
    {
      if ($self->{newLot} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = "SecurityEventOjbect->isValid(): Invalid new lot (" . $self->{newLot} . ")!  Lot does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "oldLot")
    {
      if ($self->{oldLot} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = "SecurityEventOjbect->isValid(): Invalid old lot (" . $self->{oldLot} . ")!  Lot does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "amount")
    {
      if ($self->{amount} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty amount!  The amount must be specified.";
        return 0;
      }
      if ($self->{amount} !~ /^(-)?(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid amount (" . $self->{amount} . ")!  Amount does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "newShares")
    {
      if ($self->{newShares} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty new shares!  The new shares must be specified.";
        return 0;
      }
      if ($self->{newShares} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid new shares (" . $self->{newShares} . ")!  New shares does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "oldShares")
    {
      if ($self->{oldShares} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty old shares!  The old shares must be specified.";
        return 0;
      }
      if ($self->{oldShares} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid old shares (" . $self->{oldShares} . ")!  Old shares does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "shares")
    {
      if ($self->{shares} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty shares!  The shares must be specified.";
        return 0;
      }
      if ($self->{shares} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid shares (" . $self->{shares} . ")!  Shares does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "commission")
    {
      if ($self->{commission} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid commission (" . $self->{commission} . ")!  Commission does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "costPercentage")
    {
      if ($self->{costPercentage} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid cost percentage (" . $self->{costPercentage} . ")!  Cost percentage does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "price")
    {
      if ($self->{price} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid price (" . $self->{price} . ")!  Price does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "ledgerCode")
    {
      if ($self->{ledgerCode} eq "")
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Empty ledger code!  The ledger code must be specified.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "eventRefNum")
    {
      if ($self->{eventRefNum} eq "" || $self->{eventRefNum} !~ /^[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid event reference number (" . $self->{eventRefNum} . ")!  The event reference number does not appear to be numeric.";
        return 0;
      }
    }
    elsif ($validate_list[$validatecount] eq "masterEventRef")
    {
      if ($self->{masterEventRef} eq "" || $self->{masterEventRef} !~ /^-?[0-9]+$/)
      {
        $self->{error} = 1;
        $self->{errorString} = $errorHeader . "Invalid master event reference number (" . $self->{masterEventRef} . ")!  The master event reference number does not appear to be numeric.";
        return 0;
      }
    }
  }
  
  return 1;
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

# display
# requires: stocks, accounts (hash refs that map number to name), displayObj DisplayItem
# optional: printable, codeFontSize, normalFontSize
# returns: string representing the HTML table that represents this SecurityEvent Object
# summary: This routine provides a transparent method of displaying the contents of this SecurityEvent Object.  The output will generate it's own table that needs
# to be embedded in another table as the caller sees fit to format it within their output.
sub display
{
  my $self = shift;
  my %args = ( stocks => undef, accounts => undef, codeFontSize => "-1", normalFontSize => "+0", @_ );
  my $printable = (exists $args{printable} ? $args{printable} : "false");
  my $errStr = "Portal::Accounting::Objects::SecurityEventObject->display()  - Error!<br>\n";
  my $output = "";
  my $variables = Portal::Accounting::Data::Variables->new;
  my $stocks = $args{stocks};
  my $accounts = $args{accounts};
  my $displayObj = $args{displayObj};
  my $codeFontSize = $args{codeFontSize};
  my $normalFontSize = $args{normalFontSize};

  if (! defined $stocks)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "stocks must be defined!<br>\n";
    return $output;
  }
  if (! defined $accounts)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "accounts must be defined!<br>\n";
    return $output;
  }
  if (! defined $displayObj)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "displayObj must be defined!<br>\n";
    return $output;
  }
  
  if ($printable !~ /^(true|false)$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errStr . "printable = '$printable' is invalid!<br>\n";
    return $output;
  }
  
  $output .= "<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\" width=\"100%\">\n";
  $output .= "  <tr>\n";
  
  my $eventCode = $variables->{investmentNumberToCodeHash}->{$self->{eventCode}};
  # display Event Code -> Name
  $output .= "    <td align=\"left\" width=\"7%\"" . ($eventCode eq "swap" || $eventCode eq "sxfr" ? " rowspan=\"2\"" : "") . "><font size=\"$normalFontSize\"><b>$eventCode</b></font></td>\n";
  
  # display Calc Date  
  $output .= "    <td align=\"center\" width=\"12%\"" . ($eventCode eq "swap" || $eventCode eq "sxfr" ? " rowspan=\"2\"" : "") . "><font size=\"$normalFontSize\">$self->{calcDate}</font></td>\n";
  
  # display Ledger Code
  $output .= "    <td align=\"left\" width=\"5%\"" . ($eventCode eq "swap" || $eventCode eq "sxfr" ? " rowspan=\"2\"" : "") . "><font size=\"$codeFontSize\"><b>L:</b></font> <font size=\"$normalFontSize\">$self->{ledgerCode}</font></td>\n";
  
  # display the Security (subaccount)
  $output .= "    <td align=\"left\" width=\"10%\"><font size=\"$codeFontSize\"><b>S:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum}}</font></td>\n";
  
  if ($eventCode eq "buy" || $eventCode eq "sell")
  {
    # account
    $output .= "    <td align=\"left\" width=\"10%\"><font size=\"$codeFontSize\"><b>A:</b></font> <font size=\"$normalFontSize\">$accounts->{$self->{accountNum}}</font></td>\n";
    # price
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>P:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => sprintf("%.2f", $self->{price})) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # shares
    (my $tmp = $self->{shares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Sh:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # commission
    $output .= "    <td align=\"left\" width=\"15%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>C:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => sprintf("%.2f", $self->{commission})) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # lot
    $output .= "    <td align=\"left\" width=\"8%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Lo:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">$self->{lot}</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # sub entity
    $output .= "    <td align=\"left\" width=\"5%\">" . ($eventCode eq "buy" ? "<font size=\"$codeFontSize\"><b>Se:</b></font> <font size=\"$normalFontSize\">$self->{subEntity}</font>" : "&nbsp;") . "</td>\n";
  }
  elsif ($eventCode eq "splt" || $eventCode eq "sdiv" || $eventCode eq "acqs")
  {
    if ($eventCode ne "acqs")
    {
      $output .= "    <td align=\"left\" width=\"24%\">&nbsp;</td>\n";
    }
    else
    {
      # security 2
      $output .= "    <td align=\"left\" width=\"24%\"><font size=\"$codeFontSize\"><b>S2:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum2}}</font></td>\n";
    }
    # old shares
    (my $tmp = $self->{oldShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>OS:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # new shares
    (my $tmp = $self->{newShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"15%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>NS:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # close out
    $output .= "    <td align=\"left\" width=\"13%\">&nbsp;</td>\n";
  }
  elsif ($eventCode eq "memo")
  {
    $output .= "    <td align=\"left\" width=\"66%\"><font size=\"$normalFontSize\">$self->{memoText}</font></td>\n";
  }
  elsif ($eventCode eq "spof" || $eventCode eq "swap")
  {
    # security 2
    $output .= "    <td align=\"left\" width=\"24%\"><font size=\"$codeFontSize\"><b>S2:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum2}}</font></td>\n";
    # old shares
    (my $tmp = $self->{oldShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>OS:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # new shares
    (my $tmp = $self->{newShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"15%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>NS:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # close out
    $output .= "    <td align=\"left\" width=\"13%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>C%:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => sprintf("%.2f", $self->{costPercentage}*100)) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    
    if ($eventCode eq "swap")
    {
      # close out the row and start the next row.
      $output .= "  </tr>\n";
      $output .= "  <tr>\n";
      # display security 2
      $output .= "    <td align=\"left\" width=\"10%\"><font size=\"$codeFontSize\"><b>S:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum2}}</font></td>\n";
      # security 1
      $output .= "    <td align=\"left\" width=\"24%\"><font size=\"$codeFontSize\"><b>S2:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum}}</font></td>\n";
      # new shares
      (my $tmp = $self->{newShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
      $output .= "    <td align=\"left\" width=\"14%\">\n";
      $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
      $output .= "        <tr>\n";
      $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>NS:</b></font></td>\n";
      $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
      $output .= "        </tr>\n";
      $output .= "      </table>\n";
      $output .= "    </td>\n";
      # old shares
      (my $tmp = $self->{oldShares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
      $output .= "    <td align=\"left\" width=\"15%\">\n";
      $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
      $output .= "        <tr>\n";
      $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>S:</b></font></td>\n";
      $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
      $output .= "        </tr>\n";
      $output .= "      </table>\n";
      $output .= "    </td>\n";
      # close out
      $output .= "    <td align=\"left\" width=\"13%\">\n";
      $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
      $output .= "        <tr>\n";
      $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>C%:</b></font></td>\n";
      $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => sprintf("%.2f", (1-$self->{costPercentage})*100)) . "</font></td>\n";
      $output .= "        </tr>\n";
      $output .= "      </table>\n";
      $output .= "    </td>\n";
    }
  }
  elsif ($eventCode eq "sxfr")
  {
    # old account
    $output .= "    <td align=\"left\" width=\"24%\"><font size=\"$codeFontSize\"><b>OA:</b></font> <font size=\"$normalFontSize\">$accounts->{$self->{oldAccountNum}}</font></td>\n";
    # -shares
    (my $tmp = $self->{shares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Sh:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">-" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # commission
    $output .= "    <td align=\"left\" width=\"15%\">&nbsp;</td>\n";
    # old lot
    $output .= "    <td align=\"left\" width=\"8%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>OL:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">$self->{oldLot}</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # close out
    $output .= "    <td align=\"left\" width=\"5%\">&nbsp;</td>\n";
    $output .= "  </tr>\n";
    $output .= "  <tr>\n";
    # display security
    $output .= "    <td align=\"left\" width=\"10%\"><font size=\"$codeFontSize\"><b>S:</b></font> <font size=\"$normalFontSize\">$stocks->{$self->{subAccountNum}}</font></td>\n";
    # new account
    $output .= "    <td align=\"left\" width=\"24%\"><font size=\"$codeFontSize\"><b>NA:</b></font> <font size=\"$normalFontSize\">$accounts->{$self->{newAccountNum}}</font></td>\n";
    # shares
    (my $tmp = $self->{shares}) =~ s/^(\d+\.\d[1-9]*)(0+)$/$1/;
    $output .= "    <td align=\"left\" width=\"14%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Sh:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => $tmp) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # commission
    $output .= "    <td align=\"left\" width=\"15%\">&nbsp;</td>\n";
    # new lot
    $output .= "    <td align=\"left\" width=\"8%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>NL:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">$self->{newLot}</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # close out
    $output .= "    <td align=\"left\" width=\"5%\">&nbsp;</td>\n";
  }
  elsif ($eventCode eq "fee" || $eventCode eq "rtcp")
  {
    # account
    $output .= "    <td align=\"left\" width=\"24%\">" . ($eventCode eq "fee" ? "<font size=\"$codeFontSize\"><b>A:</b></font> <font size=\"$normalFontSize\">$accounts->{$self->{accountNum}}</font>" : "&nbsp;") . "</td>\n";
    # filler
    $output .= "    <td align=\"left\" width=\"14%\">&nbsp;</td>\n";
    # amount
    $output .= "    <td align=\"left\" width=\"15%\">\n";
    $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
    $output .= "        <tr>\n";
    $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Am:</b></font></td>\n";
    $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">" . $displayObj->commify(value => sprintf("%.2f", $self->{amount})) . "</font></td>\n";
    $output .= "        </tr>\n";
    $output .= "      </table>\n";
    $output .= "    </td>\n";
    # lot
    if ($eventCode eq "fee")
    {
      $output .= "    <td align=\"left\" width=\"8%\">\n";
      $output .= "      <table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\">\n";
      $output .= "        <tr>\n";
      $output .= "          <td align=\"left\" width=\"20%\"><font size=\"$codeFontSize\"><b>Lo:</b></font></td>\n";
      $output .= "          <td align=\"right\" width=\"80%\"><font size=\"$normalFontSize\">$self->{lot}</font></td>\n";
      $output .= "        </tr>\n";
      $output .= "      </table>\n";
      $output .= "    </td>\n";
    }
    elsif ($eventCode eq "rtcp")
    {
      $output .= "    <td align=\"left\" width=\"8%\">&nbsp;</td>\n";
    }
    # close out
    $output .= "    <td align=\"left\" width=\"5%\">&nbsp;</td>\n";
  }

  $output .= "  </tr>\n";
  $output .= "</table>\n";

  return $output;
}

1;
__END__

=head1 NAME

SecurityEventObject - Object used to build a security event that can be used for database transactions.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SecurityEventObject;
  my $secobj = SecurityEventObject->new(eventCode => 'buy',
                                        calcDate => '06-29-2000',
                                        transDate => '06-29-2000',
                                        accountNum => 23470,
                                        subAccountNum => 1500,
                                        lot => 12,
                                        shares => 5000,
                                        commission => '25.00',
                                        price => '15000.00',
                                        ledgerCode => 'pxi');
  if ($secobj->didErrorOccur())
  {
    die $secobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $secobj->eventCode("memo");
  $secobj->memoText("Transfer of stock happened Tuesday.");

  # make sure the object is useful by calling the validation routine
  # before writing it to the database
  if ($secobj->isValid())
  {
    updateTransaction($secobj);
  }
  else
  {
    die $secobj->errorMessage();
  }

=head1 DESCRIPTION

SecurityEventObject is a class that stores data for a security event
transaction.  It provides a container for a security event that will
be used to write to the database or update an existing record in the
database.

=head1 Exported FUNCTIONS

  scalar new(eventCode, calcDate, transDate, counter, subEntity,
             accountNum, newAccountNum, oldAccountNum, memoText,
             subAccountNum, subAccountNum2, lot, amount,
             newShares, oldShares, shares, commission,
             costPercentage, price, ledgerCode, eventRefNum)
    Creates a new instance of the
    Portal::Accounting::Objects::SecurityEventObject
    object.  A new instance can be created in an invalid state
    (i.e., missing fields, etc.).  isValid() must be called on
    the object before using the object in a database transaction.

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function should be called before
    using this object in a database transaction.

  scalar display(stocks, accounts, displayObj [, printable]
                 [, codeFontSize][, normalFontSize])
    Returns a string representing the HTML table for the
    'this' SecurityEvent object.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut
