# SubAccount.pm - The Object Class that provides a SubAccount Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: SubAccount.pm,v 1.3 2005/01/24 22:27:05 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SubAccount;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

=head1 NAME

SubAccount - Object used to create, edit, save an SubAccount.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SubAccount;
  my $obj = Portal::Accounting::Objects::SubAccount->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

SubAccount is a SubAccount class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::SubAccount module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( type => 50, @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  $self->selectTypes();
  $self->{vars}  = { foo => "string" };
  $self->{usingForms} = 1;
  my $l = $self->{langObj};
  my $Submit = $l->map("Save");
  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "SubAccounts" },
    "command" => { -Type => "hidden", -Value => "save" },
    "new" => { -Type => "hidden", -Value => "1" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 15 },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 10 , -Value => $self->suggestCode() },
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -Value => "-1" },
    "type" => { -Label => $l->map("Type"), -Value => $args{type}, -Type => "hidden" },
    "typeName" => { -Label => $self->{metaTypes}->{$args{type}}, -Type => "hidden" },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=new#
  <table class="main">
    <tr>
      <td>#LABEL=name#</td><td>#FIELD=name#</td>
    </tr><tr>
      <td>#LABEL=code#</td><td>#FIELD=code#</td>
    </tr><tr>
      <td>#LABEL=type#</td><td>#LABEL=typeName# #FIELD=type#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code id ) ],
    optional => [  ],
    constraints => {
      id => qr/^\d\d+$/,
    },
  };
#   $self->{formObj} = HTMLObject::Form->new();
  $self->{defaultForm} = "main";
#   $self->{action} = "";
#   $self->{method} = "post";
  ##db infrastructure
  $self->{id} = "id";
  $self->{INSERTorder}[0] = [ qw( sub_acc_name code type ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { sub_acc_number => "", type => "" };
  $self->{db2data}[0] = { sub_acc_number => 'id', sub_acc_name => 'name' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM sub_acc_tb WHERE sub_acc_number = '#FIELD=$self->{id}#'";
  $self->{INSERT}[0] = "INSERT INTO sub_acc_tb (sub_acc_number, ##COLUMNS##) VALUES (next_sub_account(), ##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE sub_acc_tb SET ##COLUMNS## WHERE sub_acc_number = '#FIELD=$self->{id}#' AND NOT type IN (1, 15, 90)";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM sub_acc_tb WHERE sub_acc_number = #FIELD=$self->{id}# AND NOT type IN (1, 15, 90)";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  $self->{preDELETE}[0] = "SELECT debitcredit_tb.parent_num FROM debitcredit_tb WHERE debitcredit_tb.sub_account_num = '#FIELD=$self->{id}#' ";
  my $Reserved = $l->map("Reserved Type");
  my $cannot = $l->map("Cannot Delete");
  $self->{preDELETE}[1] = "SELECT '$Reserved'||':'||'$cannot' WHERE '#FIELD=type#' IN (1, 15, 90)";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[1] = "SELECT sub_acc_name||', '||code FROM sub_acc_tb WHERE sub_acc_number = '#FIELD=$self->{id}#'";

  return $self;
}

=item string suggestCode()

=cut
sub suggestCode
{
  my $self = shift;
  my $sql = "SELECT next_sub_account() AS n";
  my $code;
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  if (my $data = $result->fetchrow_hashref)
  {
    $code = $data->{n};
  }
  return $code;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean selectTypes()

  Routine loads types from sub account type tb into
  object under $self->{metaTypes}

=cut
sub selectTypes
{
  my $self = shift;
  my $records = 0;
  my $l = $self->{langObj};
  $self->{metaTypes} = {  };
  my $sql = "SELECT name, num FROM sub_acct_type_tb";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  while (my @data = $result->fetchrow_array)
  {
    $records++;
    $self->{metaTypes}->{$data[1]} = $l->map($data[0]);
  }
  return $records;
}

sub select
{
  my $self = shift;
  if (!$self->SUPER::select(@_))
  {
    $self->prefixError();
    return $self;
  }
  $self->{data}->{main}->{typeName}->{-Value} = $self->{metaTypes}->{$self->{data}->{main}->{type}->{-Value}};
}

=item string typeName

  Returns the name in the correct language for the type of this sub account.

=cut
sub typeName
{
  my $self = shift;
  return $self->{metaTypes}->{$self->{data}->{main}->{type}->{-Value}};
}

=item integer type

  Returns the type for this object.

=cut
sub type
{
  my $self = shift;
  return $self->{data}->{main}->{type}->{-Value};
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
