# SubEntity.pm - The Object Class that provides a SubEntity Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: SubEntity.pm,v 1.2 2005/01/24 23:27:30 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::SubEntity;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.2 $';

=head1 NAME

SubEntity - Object used to create, edit, save an SubEntity.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::SubEntity;
  my $obj = Portal::Accounting::Objects::SubEntity->new();
  $obj->populate(
    name => "test", type => 1, code => "1234", );
  if ($obj->error)
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

SubEntity is a SubEntity class.  This represents a sub account in the
accounting system.  Right now number is used internally. Name and code
are user defined.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::SubEntity module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", code => "", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  $self->{vars}  = { foo => "string" };
  $self->{usingForms} = 1;
  my $l = $self->{langObj};
  my $Submit = $l->map("Save");
  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "SubEntities" },
    "command" => { -Type => "hidden", -Value => "save" },
    "new" => { -Type => "hidden", -Value => "1" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 15 },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 5},
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=new#
  #FIELD=ledgerCode#
  <table class="main">
    <tr>
      <td>#LABEL=name#</td><td>#FIELD=name#</td>
    </tr><tr>
      <td>#LABEL=code#</td><td>#FIELD=code#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code ledgerCode ) ],
    optional => [  ],
    constraints => {   },
  };
  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{id} = "code";
  $self->{INSERTorder}[0] = [ qw( name code ledger_code ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { code => "", ledger_code => "" };
  $self->{db2data}[0] = { ledger_code => 'ledgerCode' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  $self->{INSERT}[0] = "INSERT INTO sub_entity_tb (##COLUMNS##) VALUES (##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE sub_entity_tb SET ##COLUMNS## WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  $self->{preDELETE}[0] = "SELECT debitcredit_tb.parent_num::text FROM debitcredit_tb WHERE debitcredit_tb.sub_e_code = '#FIELD=$self->{id}#' ";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[0] = "SELECT name||', '||code FROM sub_entity_tb WHERE code = '#FIELD=$self->{id}#' AND ledger_code = '#FIELD=ledgerCode#'";

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub code
{
  my $self = shift;
  my $form = "main";
  return $self->{data}->{$form}->{code}->{-Value};
}

sub name
{
  my $self = shift;
  my $form = "main";
  return $self->{data}->{$form}->{name}->{-Value};
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
