# Register.pm
#$Id: Register.pm,v 1.9 2005/01/26 22:34:58 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2004 http://www.pcxperience.org  All rights reserved.

=head1 NAME

Register - Object used to output Register screens

=head1 SYNOPSIS

  use Portal::Accounting::Register;
  my $obj = Portal::Accounting::Register->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Register is an accounting class for managing screens working
with check registers, bank reconciliations, and lists of
transactions in any given account.

=cut
package Portal::Accounting::Register;
my $state = "Register";
use strict;
use Portal::Accounting::Ledger;
use Portal::Accounting::Objects::Register;
use Portal::Accounting::Objects::RegisterForm;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Ledger Exporter AutoLoader);
@EXPORT = qw();

my $Revision = '$Revision: 1.9 $';
($VERSION = $Revision) =~ s/^Revision:\s//;

# global variables.

# modify to represent the commands this state provides.
my %commands = (
              display => "Registers",        #screen for choosing dates and which account to work with
              displayRegister => "Register",      #shows transactions with buttons for reconciliation process
              saveReconcile => "Reconcile",      #saves state of cleared entries
              reconcile => "Reconcile",      #save state of cleared entries and mark all x as r, displays summary of outstanding entries
              );

=head1 Exported FUNCTIONS


B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj)

 Creates a new instance of the Portal::Accounting::Register module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self;
  eval { $self = $class->SUPER::new(@_); };
  if ($@)
  {
    die $@;
    return $self;
  }
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # do validation
  if (!$self->Portal::Accounting::Register::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;
  return $self;
}

=head2 bool isValid(void)

    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # do validation code here.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_display
{
  my $self = shift;
  my %args = (message => "", @_);
  my %message;
  $message{body} = $args{message};
  my $doc = $self->prepDoc();
  $doc->print(%message);
  my $reg = Portal::Accounting::Objects::Register->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode}, id => $self->{userObj}->{id}, debug => 1);
  if ($reg->error) { $doc->print($reg->errorMessage); }
  $doc->print($reg->debugString);
  $doc->setFocus("body");

  if ($self->{input}->{command} eq "displayRegister")
  {
    if ($reg->populate(%{$self->{input}}) and $reg->validateForm(input => %{$self->{input}}), form => "main")
    {
      if ($reg->select)
      {
        my %foo = $reg->printHTML;
        if (defined %foo){  $doc->print(%foo);  }
        else { $doc->print($reg->errorMessage);  }
        return $doc;
      }
    }
    else
    {
      $doc->print($reg->errorMessage.$reg->debugString);
    }
  }
  my %form = $reg->printForm(form => "main");
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print("Form Failed!<br>".$reg->errorMessage.$reg->debugString);
  }
  $doc->print($reg->errorMessage);
  $doc->print($reg->debugString);
  return $doc;
}

sub c_displayRegister
{
  my $self = shift;
  return $self->c_display;
}

sub c_finishReconcile
{
  my $self = shift;
  return $self->c_reconcile;
}

=item HTMLObject c_reconcile

  what api do i want for save reconcile?

  $reg = Objects::Reconcile->new
  $reg->populate && validateForm(form => reconcile
  $reg->insert

=cut
sub c_reconcile
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $reg = Portal::Accounting::Objects::Register->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  if ($reg->populate(%{$self->{input}}) && $reg->validateForm(form => "main", input => $self->{input}) )
  {
    if (! $reg->insert()) { $doc->print($reg->errorMessage);  return $doc;  }
  }
  else { $doc->print($reg->errorMessage);  return $doc;  }
  if ($self->{input}->{command} eq "finishReconcile")
  {
    my $f = $reg->update();
    if (! defined $f) { $doc->print($reg->errorMessage);  }
    else {  $doc->print($f);  }
  }
  else
  {
    if ($reg->select)  {  $doc->print($reg->printHTML);  }
    else  {  $doc->print($reg->errorMessage);  }
  }
  return $doc;
}

1;
__END__

=head2

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT Moree
PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
