# StockPrices.pm - The Object Class that provides a StockPrices Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: StockPrices.pm,v 1.12 2004/12/22 21:34:01 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::StockPrices;
use strict;
use Portal::Accounting::Base;
use Portal::Accounting::DisplayItems;
use Portal::Accounting::DB::Main;
use Portal::Accounting::DB::StockPriceUpdate;
use LWP::Simple;
use HTML::TokeParser::Simple;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.03';

# global variables.
# modify to represent the commands this state provides.
my %commands = ( 
	display => "Stock Maintenance Screen", 
	lookupStockInfo => "Lookup Stock Information", 
	update => "Update Stocks Prices" 
  );

=head1 NAME

StockPrices - Object used to build a StockPrices Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::StockPrices;
  my $obj = Portal::Accounting::StockPrices->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

StockPrices is a StockPrices class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::StockPrices module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject run(command)

    Runs the specified command and returns the resulting HTMLObject.
=cut

sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $ledger;
  my $ledgerCode;

  if (!exists $commands{$command})
  {
    $self->setError(sprintf($self->{langObj}->map("commandNotKnown"), $command));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    $doc = $self->displayStockPriceScreen(doc => $doc);
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }
  elsif ($command eq "lookupStockInfo")
  {
    $doc->setTitle($self->{langObj}->map("chooseStockInfo"));
    $doc->setLinkDecorations;

    $doc->setFocus("body");
    $doc->print(<<"END_OF_CODE");
  <table border="0" cellspacing="0" cellpadding="0">
    <tr>
      <td bgcolor="#bcbcbc">
        <table border="0" cellspacing="1" cellpadding="3">
END_OF_CODE
    my $result = 0;
    if ((length $self->{input}->{subAccountName} > 0) || (length $self->{input}->{ticker} > 0) || (length $self->{input}->{stockName} > 0))
    {
      $result = $self->lookupStockInfo(doc => $doc, info => join('+', $self->{input}->{subAccountName}, $self->{input}->{ticker}, $self->{input}->{stockName}));
      if ($result == 0 || $self->didErrorOccur)
      {
        $doc->print(sprintf($self->{langObj}->map("errorSearchingStockInfo"), $self->{input}->{subAccountName}, $self->{input}->{ticker}, $self->{input}->{stockName}) . "<br>\n" . $self->errorMessage);
      }
    }
    else
    {
      $doc->print("<tr><td bgcolor=\"#ffffff\"><a href=\"\" onClick=\"window.close(); return false;\">" . $self->{langObj}->map("noStockInfoSuppliedReturned") . "</a></td></tr>\n");
    }
    $doc->print(<<"END_OF_CODE");
        </table>
      </td>
    </tr>
  </table>
END_OF_CODE
  }
  elsif ($command eq "update")
  {
    my $subCommand = $self->{input}->{subCommand};
    my $stocks = $self->{input}->{stocks};  # the selected stocks to work with if command is currentPrice or historicalPrice
    my $manualPrice = $self->{input}->{manualPrice};
	  if ($stock eq "") {  next;  }
    my $startDate = $self->{input}->{startDate};
    my $endDate = $self->{input}->{endDate};
    my $interval = $self->{input}->{interval};
    $ledger = $self->{input}->{ledger};
    $ledgerCode = $self->{input}->{ledgerCode};#$self->{sessionObj}->{store}->{ledgers}->{$ledger}->code;
    my $dbObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
    if ($dbObj->didErrorOccur)
    {
      $self->setError(errorString => $dbObj->errorMessage);
      return undef;
    }

    if ($subCommand !~ /^(currentPrice|historicalPrice|manualPrice)$/)
    {
      $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "subCommand = '$subCommand' is invalid!<br>\n");
      if ($self->didErrorOccur)
	  if ($stockObj->ticker eq "" || $stockObj->market eq "None") {  next;  }
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    if ($subCommand eq "currentPrice")
    {
      # get the list of stocks to work with (sub account numbers)
      my @stocks = split /\0/, $stocks;
      my @tickers = ();
      my $tickers = "";

      if (scalar @stocks > 0)
      {
        # lookup the ticker for each specified stock
        foreach my $stock (@stocks)
	  if ($stockObj->ticker eq "" || $stockObj->market eq "None") {  next;  }
        {
          my $stockObj = $dbObj->getStockInfo(number => $stock);
          if ($dbObj->didErrorOccur)
          {
            $self->setError(errorString => $dbObj->errorMessage);
            return undef;
          }
          if (!defined $stockObj)
          {
            $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "stock = '$stock' could not be found!<br>\n");
            if ($self->didErrorOccur)
            {
              $self->prefixError();
              return undef;
            }
            return $doc;
          }
          $tickers .= ", " if (length $tickers > 0);
          $tickers .= $stockObj->ticker;
          $tickers[scalar @tickers] = $stockObj->ticker;
        }
      }
      else
      {
        # get all stocks in the system and create the array of tickers
        my @stocks = $dbObj->getListOfStocks(marketOnly => 1);
        if ($dbObj->didErrorOccur)
        {
          $self->setError(errorString => $dbObj->errorMessage);
          return undef;
        }

        foreach my $stockObj (@stocks)
        {
          $tickers .= ", " if (length $tickers > 0);
          $tickers .= $stockObj->ticker;
          $tickers[scalar @tickers] = $stockObj->ticker;
        }
      }

      # instantiate the helper module to lookup prices on the web.
      my $spObj = Portal::Accounting::DB::StockPriceUpdate->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});

      # have it lookup the prices and store the results.
      (my $result, my $goodTickers, my $badTickers) = $spObj->updateStockPrices(tickerList => \@tickers, type => "current");
      if ($spObj->didErrorOccur)
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => $spObj->errorMessage);
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      # return to the main screen.
      $doc = $self->displayStockPriceScreen(doc => $doc, statusString => "Current Prices gathered for ticker(s) <b>" . join(", ", @{$goodTickers}) . "</b>" . (@{$badTickers} > 0 ? ";<br><br>Current Prices not gathered for ticker(s) <b>" . join(", ", @{$badTickers}) . "</b>.<br><br>" : ".") . "<br>\n") if ($result);
      $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "Failed to gather Current Prices for ticker(s) <b>$tickers</b>.<br>\n") if (!$result);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
    }
    elsif ($subCommand eq "historicalPrice")
    {
      my @dateResult = $self->{methods}->fixupDate(date => $startDate);
      if (!$dateResult[0])
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "startDate = '$startDate' is invalid!<br>$dateResult[2]\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }
      else
      {
        # make sure the date is "fixed".
        $startDate = $dateResult[1];
      }
      @dateResult = $self->{methods}->fixupDate(date => $endDate);
      if (!$dateResult[0])
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "endDate = '$endDate' is invalid!<br>$dateResult[2]\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }
      else
      {
        # make sure the date is "fixed".
        $endDate = $dateResult[1];
      }
      if ($interval !~ /^(day|week|month)$/)
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "interval = '$interval' is invalid!<br>\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      # get the list of stocks to work with (sub account numbers)
      my @stocks = split /\0/, $stocks;
      my @tickers = ();
      my $tickers = "";

      if (scalar @stocks > 0)
      {
        # lookup the ticker for each specified stock
        foreach my $stock (@stocks)
        {
          my $stockObj = $dbObj->getStockInfo(number => $stock);
          if ($dbObj->didErrorOccur)
          {
            $self->setError(errorString => $dbObj->errorMessage);
            return undef;
          }
          if (!defined $stockObj)
          {
            $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "stock = '$stock' could not be found!<br>\n");
            if ($self->didErrorOccur)
            {
              $self->prefixError();
              return undef;
            }
            return $doc;
          }
          $tickers .= ", " if (length $tickers > 0);
          $tickers .= $stockObj->ticker;
          $tickers[scalar @tickers] = $stockObj->ticker;
        }
      }
      else
      {
        # get all stocks in the system and create the array of tickers
        my @stocks = $dbObj->getListOfStocks(marketOnly => 1);
        if ($dbObj->didErrorOccur)
        {
          $self->setError(errorString => $dbObj->errorMessage);
          return undef;
        }

        foreach my $stockObj (@stocks)
        {
          $tickers .= ", " if (length $tickers > 0);
          $tickers .= $stockObj->ticker;
          $tickers[scalar @tickers] = $stockObj->ticker;
        }
      }

      # instantiate the helper module to lookup prices on the web.
      my $spObj = Portal::Accounting::DB::StockPriceUpdate->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});

      # have it lookup the prices and store the results.
      (my $result, my $goodTickers, my $badTickers) = $spObj->updateStockPrices(tickerList => \@tickers, type => "historical", startDate => $startDate, endDate => $endDate, interval => $interval);
      if ($spObj->didErrorOccur)
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => $spObj->errorMessage);
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      # return to the main screen.
      $doc = $self->displayStockPriceScreen(doc => $doc, statusString => "Historical Prices gathered for ticker(s) <b>" . join(", ", @{$goodTickers}) . "</b>" . (@{$badTickers} > 0 ? ";<br><br>Historical Prices not gathered for ticker(s) <b>" . join(", ", @{$badTickers}) . "</b><br><br>" : "") . "; from <b>$startDate</b> to <b>$endDate</b> by <b>$interval</b>.<br>\n") if ($result);
      $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "Failed to gather Historical Prices for ticker(s) <b>$tickers</b>; from <b>$startDate</b> to <b>$endDate</b> by <b>$interval</b>.<br>\n") if (!$result);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
    }
    elsif ($subCommand eq "manualPrice")
    {
      if (not defined $manualPrice || $manualPrice eq "")
      {
        my $manualPriceNeedsToBeSpecifiedPhrase = $self->{langObj}->map("manualPriceNeedsToBeSpecified");
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "$manualPriceNeedsToBeSpecifiedPhrase<br>\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      # get the list of stocks to work with (sub account numbers)
      my @stocks = split /\0/, $stocks;
      my @tickers = ();
      my $tickers = "";

      if (scalar @stocks > 0)
      {
        # lookup the ticker for each specified stock
        foreach my $stock (@stocks)
        {
          my $stockObj = $dbObj->getStockInfo(number => $stock);
          if ($dbObj->didErrorOccur)
          {
            $self->setError(errorString => $dbObj->errorMessage);
            return undef;
          }
          if (!defined $stockObj)
          {
            $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "stock = '$stock' could not be found!<br>\n");
            if ($self->didErrorOccur)
            {
              $self->prefixError();
              return undef;
            }
            return $doc;
          }
          $tickers .= ", " if (length $tickers > 0);
          $tickers .= $stockObj->ticker;
          $tickers[scalar @tickers] = $stockObj->ticker;
        }
      }
      else
      {
        # output an error message stating you have to choose stocks to update
	my $chooseStocksManuallyPhrase = $self->{langObj}->map("chooseStocksManually");
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "$chooseStocksManuallyPhrase<br>\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      my @dateResult = $self->{methods}->fixupDate(date => $startDate);
      if (!$dateResult[0])
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "startDate = '$startDate' is invalid!<br>$dateResult[2]\n");
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }
      else
      {
        # make sure the date is "fixed".
        $startDate = $dateResult[1];
      }

      # instantiate the helper module to lookup prices on the web.
      my $spObj = Portal::Accounting::DB::StockPriceUpdate->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});

      # have it lookup the prices and store the results.
      (my $result, my $goodTickers, my $badTickers) = $spObj->updateStockPrices(tickerList => \@tickers, type => "manual", manualPrice => $manualPrice, startDate => $startDate);
      if ($spObj->didErrorOccur)
      {
        $doc = $self->displayStockPriceScreen(doc => $doc, errorString => $spObj->errorMessage);
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }

      # return to the main screen.
      $doc = $self->displayStockPriceScreen(doc => $doc, statusString => "Manual Prices set for ticker(s) <b>" . join(", ", @{$goodTickers}) . "</b> for date $startDate.<br>\n") if ($result);
      $doc = $self->displayStockPriceScreen(doc => $doc, errorString => "Failed to set Manual Prices for ticker(s) <b>$tickers</b>.<br>\n") if (!$result);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
    }
  }

  return $doc;
}

# displayStockPriceScreen
# requires: doc
# optional: errorString, statusString
# returns: doc
# summary: This routine displays all stocks in the system and provides an interface
# to get current stock prices or get historical stock prices or manually set prices.
sub displayStockPriceScreen
{
  my $self = shift;
  my %args = ( doc => undef, errorString => "", statusString => "", @_ );
  my $doc = $args{doc};
  my $errorString = $args{errorString};
  my $statusString = $args{statusString};
  my $ledger = $self->{input}->{ledger};
  my $startDate = $self->{methods}->getDateOffset(date => "now", offset => "1 year ago", format => "%Y-%m-%d");
  my $tmpDate = $self->{methods}->getCurrentDate();
  my $dateYear = $self->{methods}->getCurrentDate(format => "%Y");
  my $ledgerCode = $self->{input}->{ledgerCode};#sessionObj}->{store}->{ledgers}->{$ledger}->code;
  my $dbObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  if (!defined $doc)
  {
    $self->setError(errorString => "doc must be defined!<br>\n");
    return undef;
  }

  # get list of All Stocks in the System with Tickers
  my @stocks = $dbObj->getListOfStocks(marketOnly => 1, orderBy => "name, ticker");
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  # get list of All stocks in the System without Tickers
  my @tickerlessStocks = $dbObj->getListOfStocks(marketOnly => 1);
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  my $ledgerURI = $doc->encodeString(string => $ledger);

  my $title = $self->{langObj}->map("stockPriceTitle");
  my $execute = $self->{langObj}->map("portfolioExecute");
  my $investmentAccounts = $self->{langObj}->map("investmentAccounts");
  my $stocksPhrase = $self->{langObj}->map("stocks");
  my $manualPricePhrase = $self->{langObj}->map("Manual_Price");
  my $dateInfoPhrase = $self->{langObj}->map("dateInfo");
  my $portfolioPhrase = $self->{langObj}->map("Portfolio");
  my $datePhrase = $self->{langObj}->map("date");
  my $selectAll = $self->{langObj}->map("selectAll");
  my $unSelectAll = $self->{langObj}->map("unSelectAll");
  my $refresh = $self->{langObj}->map("refresh");
  my $resetPhrase = $self->{langObj}->map("reset");
  my $updateManualStockPricePhrase = $self->{langObj}->map("updateManualStockPrice");
  my $negativeNumberFormatPhrase = $self->{langObj}->map("negativeNumberFormatPhrase");
  my $goPhrase = $self->{langObj}->map("go");
  my $herePhrase = $self->{langObj}->map("here");
  my $noStocksCreatedPhrase = $self->{langObj}->map("noStocksCreated");
  my $noTickeredStocksCreatedPhrase = $self->{langObj}->map("noTickeredStocksCreated");
  my $toCreateStocksPhrase = $self->{langObj}->map("toCreateStocks");
  my $toMaintainStocksPhrase = $self->{langObj}->map("toMaintainStocks");
  my $maintainStocksPhrase = $self->{langObj}->map("maintainStocks");
  my $updateCurrentStockPricePhrase = $self->{langObj}->map("updateCurrentStockPrice");
  my $updateHistoricalStockPricePhrase = $self->{langObj}->map("updateHistoricalStockPrice");
  my $dayPhrase = $self->{langObj}->map("day");
  my $weekPhrase = $self->{langObj}->map("week");
  my $monthPhrase = $self->{langObj}->map("month");
  my $startDatePhrase = $self->{langObj}->map("startDate");
  my $endDatePhrase = $self->{langObj}->map("endDate");
  my $intervalPhrase = $self->{langObj}->map("interval");
  my $stockPriceInstructionsPhrase = $self->{langObj}->map("stockPriceInstructions");

  my $displayItemObj = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});
  my $siteDir = $self->{configObj}->{siteDir};

  $doc->setTitle($title);
  $doc->setLinkDecorations;

  # display the reference bar (portfolio, refresh, etc.)
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=Portfolio&command=display&ledger=$ledgerURI&ledgerCode=$ledgerCode" onMouseOver="window.status='$portfolioPhrase'">$portfolioPhrase</a>&nbsp;&nbsp;|&nbsp;
<a href=\"$siteDir/js/Accounting/accounting/cgi-bin/sub_accounts_config2.cgi?ledger=$ledgerURI&really_in_ledger=yes\" onMouseOver=\"window.status='$maintainStocksPhrase'\">$maintainStocksPhrase</a>&nbsp;&nbsp;|&nbsp;
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=StockPrices&command=display&ledger=$ledgerURI&ledgerCode=$ledgerCode" onMouseOver="window.status='$refresh'">$refresh</a>
<br>
<hr>
END_OF_CODE
  
  # do some quick validity checks.
  if (scalar @stocks == 0 && scalar @tickerlessStocks == 0)
  {
    $doc->print("$noStocksCreatedPhrase<br>$goPhrase <a href=\"$siteDir/js/Accounting/accounting/cgi-bin/sub_accounts_config3.cgi?ledger=$ledgerURI&submitted_by=createStock&num_sub_accounts=1\" onMouseOver=\"window.status='$herePhrase'\">$herePhrase</a> $toCreateStocksPhrase\n");
    return $doc;
  }
  elsif (scalar @stocks == 0 && scalar @tickerlessStocks > 0)
  {
    $doc->print("$noTickeredStocksCreatedPhrase<br>$goPhrase <a href=\"$siteDir/js/Accounting/accounting/cgi-bin/sub_accounts_config2.cgi?ledger=$ledgerURI&really_in_ledger=yes\" onMouseOver=\"window.status='$herePhrase'\">$herePhrase</a> $toMaintainStocksPhrase\n");
    return $doc;
  }

  my %tickers = ();  # hash to store subaccount number -> ticker values.
  my %names = ();  # hash to store subaccount number -> name values.

  my %startDatePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "stockPriceForm",
                          itemName => "startDate", itemValue => $startDate, year => $dateYear,
                          phrase => $startDatePhrase, class => "menu", linkClass => "menu", width => 300, height => 250,
                          onClick => "if (document.stockPriceForm.manualPrice.value == '') { document.stockPriceForm.subCommand.options[1].selected=true; }", onChange => "if (document.stockPriceForm.manualPrice.value == '') { document.stockPriceForm.subCommand.options[1].selected=true; }");
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  my %endDatePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "stockPriceForm",
                          itemName => "endDate", itemValue => $tmpDate, year => $dateYear,
                          phrase => $endDatePhrase, class => "menu", linkClass => "menu", width => 300, height => 250,
                          onClick => "document.stockPriceForm.subCommand.options[1].selected=true;", onChange => "document.stockPriceForm.subCommand.options[1].selected=true;");
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  # generate the JavaScript hashes we will need.
  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

var stockInfo = new Array();

function setStock(ticker, name)
{
  this.ticker = ticker;
  this.name = name;
}

function manualStock()
{
  var d = document.stockPriceForm;

  if (d.manualPrice.value != "")
  {
    // update the command to execute to be the Update Manual Price
    d.subCommand.options[2].selected = true;
  }
  else
  {
    // update the command to execute to be the Update Current Price if the current command is Update Manual Price
    if (d.subCommand.options[2].selected == true)
    {
      d.subCommand.options[0].selected = true;
    }
  }
}

END_OF_CODE

  my $stocksString = "";
  foreach my $stockObj (@stocks)
  {
    $stocksString .= "          <option value=\"$stockObj->{number}\">$stockObj->{name} ($stockObj->{ticker})\n";
    $tickers{$stockObj->{number}} = $stockObj->{ticker};
    $names{$stockObj->{number}} = $stockObj->{name};
    $doc->print("stockInfo[\"$stockObj->{number}\"] = new setStock(\"$stockObj->{ticker}\", \"$stockObj->{name}\");\n");
  }

  if (length $errorString > 0)
  {
    $errorString = "<font color=\"red\">Error: </font>" . $errorString . "<br>\n";
  }
  if (length $statusString > 0)
  {
    $statusString = "<font color=\"red\">Status: </font>" . $statusString . "<br>\n";
  }

  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<br>
$statusString
$errorString
<br>
<form name="stockPriceForm" method="post" action="$siteDir/cgi-bin/index.cgi">
  <input type="hidden" name="app" value="Accounting">
  <input type="hidden" name="state" value="StockPrices">
  <input type="hidden" name="command" value="update">
  <input type="hidden" name="ledger" value="$ledger">
  <input type="hidden" name="ledgerCode" value="$ledgerCode">
  <table border="0" cellpadding="5" cellspacing="0">
    <tr>
      <th align="center">$stocksPhrase</th><th align="center">$manualPricePhrase</th><th align="center">$dateInfoPhrase</th>
    </tr>
    <tr>
      <td align="center" valign="top">
        <select name="stocks" size="10" multiple>
$stocksString
        </select>
        <br>
        <input type="button" name="selectAllStocks" value="$selectAll" onClick="javascript:selectAll(document.stockPriceForm.stocks);">
        <br>
        <input type="button" name="unSelectAllStocks" value="$unSelectAll" onClick="javascript:unSelectAll(document.stockPriceForm.stocks);">
      </td>
      <td align="center" valign="top">
        <input type="text" name="manualPrice" value="" size="20" onChange="manualStock();">
      </td>
      <td align="left" valign="top">
END_OF_CODE
  $doc->print(%startDatePicker);
  $doc->print(<<"END_OF_CODE");
        <br>
END_OF_CODE
  $doc->print(%endDatePicker);
  $doc->print(<<"END_OF_CODE");
        <br>
        $intervalPhrase <select name="interval" onChange="document.stockPriceForm.subCommand.options[1].selected=true">
          <option value="day">$dayPhrase
          <option value="week">$weekPhrase
          <option value="month">$monthPhrase
        </select>
      </td>
    </tr>
  </table>
  <select name="subCommand">
    <option value="currentPrice">$updateCurrentStockPricePhrase
    <option value="historicalPrice">$updateHistoricalStockPricePhrase
    <option value="manualPrice">$updateManualStockPricePhrase
  </select>
  <input type="submit" name="update" value="$goPhrase">&nbsp;<input type="reset" value="$resetPhrase">
  <br>
  $stockPriceInstructionsPhrase
</form>
END_OF_CODE

  if (scalar @tickerlessStocks > 0)
  {
    my $stocksString = "";
    foreach my $stockObj (@tickerlessStocks)
    {
      $stocksString .= $stockObj->name . "<br>";
    }
    $doc->print("<br>\n<br>\nThese are <u>not</u> available for price updates:<br>\n$stocksString\n");
  }

  return $doc;
}

# lookupStockInfo
# requires: doc, info
# optional:
# returns: 1 on success, 0 on failure
# summary: This routine displays all stocks found by sending the value of info to
# the financial URL and parsing the returned data apart to present a listing of
# rows with the symbol, name, market, and industry of the result of the search
sub lookupStockInfo
{
  my $self = shift;
  my %args = ( doc => undef, info => "", @_ );
  my $doc = $args{doc};
  my $info = $args{info};

  my $financeURL = "http://finance.yahoo.com/l?m=US&t=S&s=";

  # output header fields
  $doc->print("<tr bgcolor=\"#cdcdcd\"><th><font size=\"-1\">" . $self->{langObj}->map("Symbol") . "</font></th><th><font size=\"-1\">" . $self->{langObj}->map("Name") . "</font></th><th><font size=\"-1\">" . $self->{langObj}->map("Market") . "</font></th><th><font size=\"-1\">" . $self->{langObj}->map("Industry") . "</font></th></tr>\n");

  my $financeHTML = LWP::Simple::get($financeURL . $info);
  my $htmlString = HTML::TokeParser::Simple->new(\$financeHTML);
  my $numValidTokens = 0;
  my $rowString = "";
  my $ticker = "";
  my $stockName = "";
  my $market = "";
  my $industry = "";
  while (my $token = $htmlString->get_token)
  {
    if ($token->as_is =~ /Add to My Portfolio/)
    {
      my $tdCount = 0;
      my $alreadyOutput = 0;
      my $alreadyTR = 0;
      my $previousValidToken = 0;
      # move token position after the /td that closes 'Add to My Portfolio'
      while (not ($token->is_end_tag('/td')))
      {
        $token = $htmlString->get_token;
      }
      # parse any returned data
      while (not ($token->as_is =~ /View Quotes for All Above Symbols/))
      {
        $token = $htmlString->get_token;
        if ($token->is_end_tag('/td'))
        {
          if ($previousValidToken > 0)
          {
            $numValidTokens++;
            $previousValidToken = 0;
          }
          $tdCount++;
          $alreadyOutput = 0;
        }
        if (($token->is_text) && (not ($token->as_is =~ /View Quotes for All Above Symbols/)) && (not ($token->as_is =~ /^Add$/)) && (not ($token->as_is =~ /^\s$/)))
        {
          if ($alreadyOutput > 0)
          {
            if ($tdCount == 0)
            {
              $ticker .= " ";
            }
            elsif ($tdCount == 1)
            {
              $stockName .= " ";
            }
            elsif ($tdCount == 2)
            {
              $market .= " ";
            }
            elsif ($tdCount == 3)
            {
              $industry .= " ";
            }
          }
          if ($tdCount == 0)
          {
            $ticker .= $token->as_is;
          }
          elsif ($tdCount == 1)
          {
            $stockName .= $token->as_is;
          }
          elsif ($tdCount == 2)
          {
            $market .= $token->as_is;
          }
          elsif ($tdCount == 3)
          {
            $industry .= $token->as_is;
          }
          $alreadyOutput = 1;
          $previousValidToken = 1;
        }
        if ($tdCount == 5)
        {
          $rowString = $doc->td(-content => $doc->a(href => "", onclick => "window.opener.document.sub_acc_form.sub_acct_name.value = '" . $stockName . "'; window.opener.document.sub_acc_form.ticker.value = '" . $ticker . "'; window.opener.document.sub_acc_form.stockName.value = '" . $stockName . "'; window.opener.matchMarket('" . $market . "'); window.close(); return false;", -content => $doc->font(size => "-2", -content => $ticker))) .
                       $doc->td(-content => $doc->font(size => "-2", -content => $stockName)) .
                       $doc->td(-content => $doc->font(size => "-2", -content => $market)) .
                       $doc->td(-content => $doc->font(size => "-2", -content => $industry));
          $doc->print($doc->tr(bgcolor => "#ffffff", -content => $rowString) . "\n");
          $ticker = "";
          $stockName = "";
          $market = "";
          $industry = "";
          $tdCount = 0;
          $alreadyOutput = 0;
        }
      }
    }
  }

  if ($numValidTokens == 0)
  {
    $doc->print("<tr><td colspan=\"4\" bgcolor=\"#ffffff\"><a href=\"\" onClick=\"window.close(); return false;\">" . $self->{langObj}->map("noStockInfoReturned") . "</a></td></tr>\n");
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Base(3)

=cut
