# SubEntities.pm - The Object Class that provides a SubEntities Object
#$Id: SubEntities.pm,v 1.1 2005/01/24 22:26:37 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::SubEntities;
use strict;
use Portal::Accounting::Main;
use Portal::Accounting::Objects::SubEntity;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.1 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Sub Entities",
  new => "New Sub Entity",
  save => "Save Sub Entity",
  edit => "Edit Sub Entity",
  delete => "Delete Sub Entity",
  );

=head1 NAME

SubEntities - Object used to work with SubEntities

=head1 SYNOPSIS

  use Portal::Accounting::SubEntities;
  my $obj = Portal::Accounting::SubEntities->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

SubEntities displays the SubEntities Menu from which sub Entities
can be edited, created, and deleted.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::SubEntities module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the SubEntities State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;

  # do validation
  if (!$self->Portal::Accounting::SubEntities::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_new
{
  my $self = shift;
  return $self->c_edit(@_);
}

sub c_save
{
  my $self = shift;
  return $self->c_edit(@_);
}

=item HTMLObject c_edit()

  this routine is to create/edit a new Entity

=cut
sub c_edit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my %args = ( @_ );
  my $command = $self->{input}->{command};
  my $l = $self->{langObj};
  my $subEntity = Portal::Accounting::Objects::SubEntity->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  if ($subEntity->error) { $doc->print($subEntity->errorMessage); return $doc;  }
  #how do i know if this is edit or new?
  if ($self->{input}->{command} eq "save")
  { #validate user input then react
    if ($subEntity->validateForm(input => $self->{input}), formName => "main")
    {
      my $result = $subEntity->populate(%{$self->{input}});
      if (not defined $result) { $self->error($subEntity->errorMessage);  return undef;  }
      if ($result) #populate worked
      {
        if ($subEntity->isNew) #insert subEntity obj
        {
          $subEntity->insert();
          $doc->print($subEntity->debugString);
        }
        else
        {
          $subEntity->update();
        }
        if ($subEntity->error)
        {
          $doc->print("Error".$subEntity->errorMessage());
        }
        else
        {
          $doc->setFocus("body");
          $doc->print("Success");
          $doc->print($self->mainButton);
          return $doc;
        }
      }
      else { $doc->print($subEntity->genErrorString("all"));  }
    }
    #invalid so let form redisplay
    else
    {
      $doc->print($subEntity->errorMessage);
    }
  }
  elsif ($command eq "edit")
  {
    my $result = $subEntity->populate(ledgerCode => $self->{input}->{ledgerCode}, code => $self->{input}->{code});
    if ($result) { $subEntity->select(); }
    if ($subEntity->error)
    {
      $doc->print("Error".$subEntity->errorMessage());
    }
  }
  #display form
  my %form = $subEntity->printForm(formName => "main");
  if ($subEntity->error) {  $doc->print($subEntity->errorMessage); }
  else {  $doc->print(%form);  }

  $doc->print($self->mainButton);
  return $doc;
}

sub c_display
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $l = $self->{langObj};
  my $New = $l->map("New");
  my $Edit = $l->map("Edit");
  my $Name = $l->map("Name");
  my $Code = $l->map("Code");
  my $Delete = $l->map("Delete");
  my $confirm = $l->map("confirmDeleteSubEntity");
  my %foo;
  my $message = "";
  my $sql = "SELECT name, code FROM sub_entity_tb WHERE ledger_code = '$self->{input}->{ledgerCode}' ORDER BY name";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  $foo{css} .= <<"ENDOF";
  .name { text-align: left; }
  .code { text-align: left; }
  .delete { text-align: center; }
ENDOF
  $doc->print(%foo);
  $doc->setFocus("body");
  $doc->print(<<"ENDOF");
  <table width="50%" class="main">
    <tr>
      <th class="name">$Name</th>
      <th class="code">$Code</th>
      <th class="delete">$Delete</th>
    </tr>
ENDOF
  while (my $data = $result->fetchrow_hashref)
  {
    $doc->print(<<"ENDOF");
  <tr>
    <td class="name"><a title="$Edit" href="?app=$self->{app}&state=$self->{state}&command=edit&ledgerCode=$self->{input}->{ledgerCode}&code=$data->{code}">$data->{name}</a></td>
    <td class="code">$data->{code}</td>
    <td class="delete">
      <form name="delete_$data->{code}">
        <input type="hidden" name="app" value="$self->{app}">
        <input type="hidden" name="state" value="$self->{state}">
        <input type="hidden" name="command" value="delete">
        <input type="hidden" name="code" value="$data->{code}">
        <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
        <input type="hidden" name="confirmDelete" value="0">
        <input type="submit" name="delete" value="X" onclick="return deleteConfirm(this.form); ">
      </form>
    </td>
  </tr>
ENDOF
  }
  $doc->print("</table>\n\n");
  if (defined $self->{input}->{message})
  {
    $message = '<p class="message">'.$self->{input}->{message}.'</p>';
  }
  $foo{javascript} = <<"ENDOF";
function deleteConfirm(myForm)
{
  if (confirm("$confirm"))
  {
    myForm.confirmDelete.value = 1;
    return true;
  }
  return false;
}
ENDOF
  $foo{body} .= $message;
  $foo{body} .= <<"ENDOF";
    <form name="new">
      <input type="hidden" name="app" value="$self->{app}">
      <input type="hidden" name="state" value="$self->{state}">
      <input type="hidden" name="command" value="new">
      <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
      <input type="submit" name="new" value="$New">
    </form>
    <br>
ENDOF
  $foo{body} .= $self->mainButton;
  $doc->print(%foo);
  return $doc;
}

=item HTMLObject c_delete()


=cut
sub c_delete
{
  my $self = shift;
  my %args = (doc => undef, @_);
  my $l = $self->{langObj};
  my $Failed = $l->map("Failed");
  my $Success = $l->map("Success");
  my $confirm = $l->map("confirmDeleteSubEntity");
  my $doc;
  if (defined $args{doc}) { $doc = $args{doc}; }
  else { $doc = $self->prepDoc();  }
  my $subEntities = $l->map("Sub Entities");

  my $subEntity = Portal::Accounting::Objects::SubEntity->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, ledgerCode => $self->{input}->{ledgerCode});
  $subEntity->populate(ledgerCode => $self->{input}->{ledgerCode}, code => $self->{input}->{code}, name => "foo");
  if (! $subEntity->select())
  {
    $doc->print("SELECT for delete failed!<br>\n".$subEntity->errorMessage."<br>".$subEntity->debugString);
    return $doc;
  }
  if ($subEntity->error) { $doc->print($subEntity->errorMessage); return $doc;  }
  if (defined $self->{input}->{confirmDelete} && $self->{input}->{confirmDelete} eq "1")
  {#do the actual delete
    my @result = $subEntity->delete();
    if (! defined @result)
    {
      $doc->print($subEntity->errorMessage);
    }
    else
    {
      my $out = $l->map("Delete") . ": ";
      if ($result[0] == -1)
      {
        $out .= "$Failed<br>".$result[1];
      }
      else {  $out .= "$Success:".$result[0]."<br>".$result[1];  }
      $doc->print($out."<br>". $subEntity->debugString);
    }
    $doc->print($self->mainButton);
    return $doc;
  }
  else
  { #output confirm d
    my $yes = $self->{langObj}->map("Yes");
    $doc->print(<<"ENDOF");
<form name="mainForm">
  <p>$confirm<br>
  <input type="submit" name="yes" value="$yes"></p>
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="delete">
  <input type="hidden" name="confirmDelete" value="1">
  <input type="hidden" name="id" value="$self->{input}->{id}">
</form>
ENDOF
  }
  $doc->print($self->mainButton);
  return $doc;
}

sub mainButton
{
  my $self = shift;
  my $return;
  my $display = $self->{langObj}->map($self->{commands}->{display});
  $return = << "ENDOF";
  <form name="display">
    <input type="submit" name="submit" value="$display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="command" value="display">
  </form>
ENDOF
  return $return;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
